#!/usr/bin/env python
"""
creates htsint database
(1) Edit/create the configure.py file and specify the database parameters
(2) Run 'DatabaseFetch.py'
(3) Run this file.
A logfile of the output is automatically created in the 'data' directory.
Use TestDatabase.py to make sure tables were filled correctly.
There is also a unittest for the database and the examples within show some
of the different ways of interacting with the database.
For more information about interacting with databases using SQLalchemy
    http://docs.sqlalchemy.org/en/rel_0_8/orm/tutorial.html
Because gene names are added/removed/changed on a regular basis
it is best to re-run FetchDbdata.py and then this script.
"""

### make imports
import sys,os,re,time,csv
from htsint import Configure
from DatabaseTables import Base,Taxon,Gene,Uniprot,GoTerm,GoAnnotation
from DatabaseTools import db_connect, get_file_sizes,print_db_summary
from DatabaseTools import populate_taxon_table,populate_gene_table,populate_uniprot_table
from DatabaseTools import populate_go_terms, populate_go_annotations
from GeneOntologyLib import read_annotation_file,get_annotation_file,get_total_annotations

class DatabaseCreate(object):
    """
    Database population class
    """

    def __init__(self):
        """
        Constructor
        """

        self.config = Configure()

        ## ensure config is setup 
        for key in ['data','dbname']:
            if self.config.log[key] == '':
                raise Exception("You must modify the config file before running DatabaseFetch.py")

        dataDir = self.config.log['data']

        if not os.path.isdir(dataDir):
            raise Exception("Specified htsint data directory does not exist %s"%dataDir)

        self.taxaList = self.config.log['taxa']

    def run(self):
        ## prepare a log file
        fid = open(os.path.join(self.config.log['data'],'createdb.log'),'wa')
        writer = csv.writer(fid)

        def push_out(line):
            writer.writerow([line])
            print(line)

        push_out(sys.argv[0])
        push_out(time.asctime())
        push_out("Getting ready to create database...")

        ## conect to the database
        session,engine = db_connect(verbose=False)
        Base.metadata.drop_all(engine)
        Base.metadata.create_all(engine)

        push_out("Creating database with...")
        for t in Base.metadata.sorted_tables:
            push_out("\t"+t.name)

        ## determine file sizes
        timeStart = time.time()
        push_out("determining filesizes...")
        idmapCount,geneInfoCount = get_file_sizes()

        push_out("extracting taxa list...")
        totalAnnotations = get_total_annotations()
        push_out("...extraction time: %s"%time.strftime('%H:%M:%S',time.gmtime(time.time()-timeStart)))

        ## taxa table
        push_out("Populating the database taxa table")
        timeStr,addedStr = populate_taxon_table(engine)
        push_out(timeStr)
        push_out(addedStr)

        ## gene table
        push_out("Populating the database with %s genes"%(geneInfoCount))
        timeStr,addedStr = populate_gene_table(geneInfoCount,session,engine)
        push_out(timeStr)
        push_out(addedStr)
        
        ## uniprot table
        push_out("Populating the database with %s uniprot entries"%(idmapCount))
        timeStr,addedStr = populate_uniprot_table(idmapCount,session,engine)
        push_out(timeStr)
        push_out(addedStr)

        ## populate the go-terms
        push_out("Populating the database with for go terms...")
        timeStr,addedStr = populate_go_terms(engine)
        push_out(timeStr)
        push_out(addedStr)

        ## populate the go-annotations
        push_out("Populating the database with for go annotations...")
        timeStr,addedStr,ignored = populate_go_annotations(totalAnnotations,session,engine)
        push_out(timeStr)
        push_out(addedStr)
        push_out("There were %s uniprot annotations ignored"%str(ignored[0]))
        push_out("There were %s gene annotations ignored"%str(ignored[1]))

        print_db_summary()
        fid.close()

if __name__ == "__main__":
    print "Running..."
    dbc = DatabaseCreate()
    dbc.run()



import sys,os

## basic files
from version import __version__
from basedir import __basedir__

## database functions and classes
from Configure import Configure
from RunSubprocess import run_subprocess, RunSubprocess
from GeneOntology import GeneOntology
from TermDistances import TermDistances
from GeneDistances import GeneDistances
from AssembleDistances import AssembleDistances
from TaxaSummary import TaxaSummary
from GeneSetCollection import GeneSetCollection
from GeneSet import GeneSet



#!/usr/bin/env python
"""
These are the tables for the database
"""

### make imports
import sys,os,re
from sqlalchemy import Table, Column, Integer, String, MetaData, ForeignKey
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Sequence
Base = declarative_base()

class Taxon(Base):
    '''
    class that handles the ncbi taxa
    '''

    __tablename__ = 'taxa'

    id = Column(Integer, Sequence('taxon_id_seq'),primary_key=True)
    ncbi_id = Column(Integer)
    name = Column(String)
    common_name_1 = Column(String)
    common_name_2 = Column(String)
    common_name_3 = Column(String)
    
    def __init__(self,ncbi_id,name='',common_name_1='',common_name_2='',common_name_3=''):
        self.ncbi_id = ncbi_id
        self.name = name
        self.common_name_1 = common_name_1
        self.common_name_2 = common_name_2
        self.common_name_3 = common_name_3
            
    def __repr__(self):
        return "<Taxon('%s','%s','%s','%s','%s')>"%(self.ncbi_id,
                                                    self.name,
                                                    self.common_name_1,
                                                    self.common_name_2,
                                                    self.common_name_3)

def taxa_mapper(session,ncbiIdList=None,myDict={},yieldPer=5):
    """
    a function that maps ncbi_ids to taxa.id
    if a dict is provided keys must be the string of the ncbi_id
    """

    if ncbiIdList:
        ncbiIdList = dict([(i,None)for i in list(set(ncbiIdList))])

    for t in session.query(Taxon).yield_per(yieldPer):
        if ncbiIdList and not ncbiIdList.has_key(str(t)):
            continue

        myDict[str(t.ncbi_id)] = t.id

    return myDict


class Gene(Base):
    '''
    class that handles ncbi genes
    '''

    __tablename__ = 'genes'

    id = Column(Integer, Sequence('gene_id_seq'),primary_key=True)
    ncbi_id = Column(String)
    description = Column(String,nullable=False)
    symbol = Column(String)
    synonyms = Column(String)
    taxa_id = Column(Integer,ForeignKey('taxa.id'),nullable=True)
    
    def __init__(self,ncbi_id,description,symbol,synonyms,taxa_id):
        self.ncbi_id = ncbi_id
        self.description = description
        self.symbol = symbol
        self.synonyms = synonyms
        self.taxa_id = taxa_id
               
    def __repr__(self):
        return "<Gene('%s','%s','%s','%s','%s')>"%(self.ncbi_id,
                                                   self.description,
                                                   self.symbol,
                                                   self.synonyms,
                                                   self.taxa_id)

def gene_mapper(session,ncbiIdList=None,myDict={}):
    """
    a function that maps ncbi_ids to gene.id
    if a dict is provided keys must be the string of the ncbi_id
    """

    if ncbiIdList:
        ncbiIdList = dict([(i,None)for i in list(set(ncbiIdList))])

    for g in session.query(Gene).yield_per(5):
        if ncbiIdList and not ncbiIdList.has_key(g):
            continue
        myDict[str(g.ncbi_id)] = g.id

    return myDict

class Uniprot(Base):
    '''
    class that handles Uniprot Accessions
    '''

    __tablename__ = 'uniprot'

    id = Column(Integer, Sequence('uniprot_id_seq'),primary_key=True)
    uniprot_ac = Column(String)
    uniprot_entry = Column(String)
    refseq = Column(String)
    taxa_id = Column(Integer,ForeignKey('taxa.id'),nullable=True)
    gene_id = Column(Integer,ForeignKey('genes.id'),nullable=True)

    def __init__(self,uniprot_ac,uniprot_entry,refseq,taxa_id,gene_id):
        self.uniprot_ac = uniprot_ac
        self.uniprot_entry = uniprot_entry
        self.refseq = refseq
        self.taxa_id = taxa_id
        self.gene_id = gene_id

    def __repr__(self):
        return "<Uniprot('%s','%s','%s','%s','%s')>"%(self.uniprot_ac,
                                                      self.uniprot_entry,
                                                      self.refseq,
                                                      self.taxa_id,
                                                      self.gene_id)

def uniprot_mapper(session,uniprotIdList=None,myDict={},gene=False,taxa=False):
    """
    a function that maps uniprot_entry to uniprot.id
    if a dict is provided keys must be the string of the ncbi_id

    session - current sqlalchemy session
    uniprotIdList - returns all entries in the table unless a list of uniProt_entries are provided
    myDict - provides a means to expand or update an existing dictionary
    gene - appends to the returned values the database gene id
    taxa - appends to the returned values the database taxa_id

    """

    if uniprotIdList:
        uniprotIdList = dict([(i,None) for i in list(set(uniprotIdList))])

    for u in session.query(Uniprot).yield_per(5):
        if uniprotIdList and not uniprotIdList.has_key(u.uniprot_entry):
            continue

        if gene == False and taxa == False:
            myDict[str(u.uniprot_entry)] = u.id
        elif gene == True and taxa == False:
            myDict[str(u.uniprot_entry)] = {'id':u.id,'gene_id':u.gene_id}
        elif gene == False and taxa == True:
            myDict[str(u.uniprot_entry)] = {'id':u.id,'taxa_id':u.taxa_id}
        elif gene == True and taxa == True:
            myDict[str(u.uniprot_entry)] = {'id':u.id,'gene_id':u.gene_id,'taxa_id':u.taxa_id}

    return myDict

class GoTerm(Base):
    '''
    class that handles gene ontology terms
    '''

    __tablename__ = 'go_terms'

    id = Column(Integer, Sequence('go_term_id_seq'),primary_key=True)
    go_id = Column(String)
    aspect = Column(String)
    name = Column(String)
    alternate_id = Column(String)
    description = Column(String)

    def __init__(self,go_id,aspect,name,alternate_id,description):
        self.go_id = go_id
        self.aspect = aspect
        self.name = name
        self.alternate_id = alternate_id
        self.description = description
    
    def __repr__(self):
        return "<GoTerm('%s','%s','%s','%s','%s')>"%(self.go_id,
                                                     self.aspect,
                                                     self.name,
                                                     self.alternate_id,
                                                     self.description)

def goterm_mapper(session,gotermIdList=None,myDict={}):
    """
    a function that maps uniprot_id to uniprot.id
    if a dict is provided keys must be the string of the ncbi_id
    """

    if gotermIdList:
        gotermIdList = dict([(i,None)for i in list(set(gotermIdList))])

    for g in session.query(GoTerm).yield_per(5):
        if gotermIdList and not gotermIdList.has_key(g):
            continue

        myDict[g.go_id] = g.id

    return myDict

class GoAnnotation(Base):
    '''
    class that handles gene ontology annotations
    annotations may be associated with a gene id or a uniprot id
    '''

    __tablename__ = 'go_annotations'

    id = Column(Integer, Sequence('go_annotation_id_seq'),primary_key=True)
    go_term_id = Column(Integer, ForeignKey('go_terms.id'))
    evidence_code = Column(String)
    pubmed_refs = Column(String)
    uniprot_id = Column(Integer, ForeignKey('uniprot.id'),nullable=True)
    gene_id = Column(Integer, ForeignKey('genes.id'),nullable=True)
    taxa_id = Column(Integer,ForeignKey('taxa.id'),nullable=False)

    def __init__(self,go_term_id,evidence_code,pubmed_refs,uniprot_id,gene_id,taxa_id):
        self.go_term_id = go_term_id
        self.evidence_code = evidence_code
        self.pubmed_refs = pubmed_refs
        self.uniprot_id = uniprot_id
        self.gene_id = gene_id
        self.taxa_id = taxa_id

    def __repr__(self):
        return "<GoAnnotation('%s','%s','%s','%s','%s','%s')>"%(self.go_term_id,
                                                                self.evidence_code,
                                                                self.pubmed_refs,
                                                                self.uniprot_id,
                                                                self.gene_id,
                                                                self.taxa_id)



#!/usr/bin/env python
"""
Library of functions to help output data in reStructuredText format
"""

## functions 
def print_rest_table_contents(columns,items,withTrailing=True):
    """
    function needs to be turned into a class
    """

    row = "+"
    head = "+"
    for col in columns:
        row += "-"*col+"-+"
        head += "="*col+"=+"

    if len(columns) != len(items):
        raise Exception("Dimension mismatch %s %s"%(len(columns),len(items)))

    toPrint = "| "

    for i,item in enumerate(items):
        item = str(item)

        if len(item) >= columns[i]:
            raise Exception("col %s not large enough min = %s"%(i,len(item)+2))

        toPrint += item+" "*(columns[i]-len(item))+"| "

    print(toPrint[:-1])

    if withTrailing:
        print(row)



#!/usr/bin/python

import time,csv,re,sys
import numpy as np
from htsint.database import get_annotation_file, get_gene2go_file


annotationFile = get_annotation_file()
annotationFid = open(annotationFile,'rU')
annotsCount = 0
annotatedIds = {}
annots1,annots2 = 0,0

for record in annotationFid:
    record = record[:-1].split("\t")

    if record[0][0] == "!":
        continue
    if record[0] != 'UniProtKB':
        continue

    uniprotEntry = record[10]
    if re.search("\|",uniprotEntry):
        uniprotEntry = re.split("\|",uniprotEntry)[0]

    #for i,r in enumerate(record):
    #    print i,r

    #if annots1 == 300:
    #    sys.exit()

    if re.search("\-1",record[1]):
        print record[1]

    taxon = re.sub("taxon:","",record[12])
    if taxon == "" or re.search("\|",taxon):
        continue

    annots1 += 1

gene2goFile = get_gene2go_file()
gene2goFid = open(gene2goFile,'rU')
header = gene2goFid.next()

for record in gene2goFid:
    annots2 += 1

print("__________________")
print("Uniprot annotations: %s"%annots1)
print("Gene2go annotations: %s"%annots2)
print("Total Annotations: %s"%(annots1 + annots2))



#!/usr/bin/env python
"""
fetch all data files for htsint
At least 20GB of free space is recommended
"""
                                                          
import os,sys,subprocess,re,time,csv
from sys import platform as _platform
from htsint import __basedir__
from htsint import Configure

class DatabaseFetch(object):
    """
    Run each time we want files updated for database
    """

    def __init__(self,wget=os.path.join("/","usr","bin","wget"),gunzip=os.path.join("/","usr","bin","gunzip")):
        """  
        Constructor
        """
 
        config = Configure()
       
        ## check that we are in linux or osx
        if _platform == "linux" or _platform == "linux2":
            pass
        elif _platform == "darwin":
            pass
        elif _platform == "win32":
            raise Exception("DatabaseFetch currently does not work on windows platforms\n"+\
                            "you may still download the files one by one and populated the db")

        ## ensure they have set up config
        for key in ['data','dbname']:
            if config.log[key] == '':
                raise Exception("You must modify the config file before running DatabaseFetch.py")

        dataDir = config.log['data']

        if not os.path.isdir(dataDir):
            raise Exception("Specified htsint data directory does not exist %s"%dataDir)

        ## move into specified directory
        self.cwd = os.getcwd()
        os.chdir(dataDir)

        ## check for the necessary programs
        for wgetPath in [wget,os.path.join('/','usr','local','bin','wget')]:
            if os.path.exists(wgetPath) == True:
                break
            wgetPath = None

        if wgetPath == None:
            raise Exception("ERROR: cannot find wget -- either download the files (see documentation) or specify a path")
        self.wgetPath = wgetPath
        
        gunzipPaths = [gunzip,os.path.join("/","bin","gunzip")]
        for gunzipPath in gunzipPaths:
            if os.path.exists(gunzipPath) == True:
                break
            gunzipPath = None

        if gunzipPath == None:
            raise Exception("ERROR: cannot find gunzip -- either download the files (see documentation) or specify a path")
        self.gunzipPath = gunzipPath

    def _run_subprocess(self,cmd):
        proc = subprocess.Popen(cmd,shell=True,stdout=subprocess.PIPE,stdin=subprocess.PIPE)
        while True:
            try:
                next_line = proc.stdout.readline()
                proc.wait()
                if next_line == '' and proc.poll() != None:
                    break
            except:
                proc.wait()
                break

    def fetch_file(self,fetchURL):
        cmd = "%s -N %s"%(self.wgetPath,fetchURL)
        self._run_subprocess(cmd)
        
    def unzip_file(self,fileName):
        print('unzipping...%s'%fileName)
        cmd = "%s -c %s > %s.db"%(self.gunzipPath,fileName,fileName[:-3])
        self._run_subprocess(cmd)

    def untar_file(self,fileName):
        print('untarring...%s'%fileName)
        cmd = "tar xzf %s"%fileName
        self._run_subprocess(cmd)

    def run(self):
        ## prepare a log file
        fid = open('fetchdb.log','w')
        writer = csv.writer(fid)

        def push_out(line):
            writer.writerow([line])
            print(line)

        push_out(sys.argv[0])
        push_out(time.asctime())
        push_out("fetching files...")

        ## fetch the files required for the database
        uniprotUrl = "ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/"
        filesToFetch = ["ftp://ftp.geneontology.org/pub/go/ontology/go.obo",
                        "ftp://ftp.ncbi.nlm.nih.gov/pub/taxonomy/taxdump.tar.gz",
                        "ftp://ftp.ncbi.nlm.nih.gov/gene/DATA/gene_info.gz",
                        "ftp://ftp.ncbi.nlm.nih.gov/gene/DATA/gene2go.gz",
                        "ftp://ftp.ebi.ac.uk/pub/databases/GO/goa/UNIPROT/gene_association.goa_uniprot.gz",
                        uniprotUrl + "idmapping/idmapping.dat.gz",
                        uniprotUrl + "idmapping/LICENSE",
                        uniprotUrl + "complete/uniprot_sprot.fasta.gz"
                    ]
                                 
        for fetchURL in filesToFetch:
            fileName = os.path.split(fetchURL)[-1]
            push_out("fetching %s..."%fileName)
            timeStart = time.time()
            self.fetch_file(fetchURL)
            fetchTime = time.time() - timeStart
            push_out("...%s"%fetchTime)

            ## unzip the gz files
            if not re.search("\.gz",fileName):
                continue

            if not os.path.exists(fileName[:-3]+".db") or fetchTime > 10:
                if re.search("\.tar\.gz",fileName):
                    self.untar_file(fileName)
                else:
                    self.unzip_file(fileName)

        push_out("complete.")
        fid.close()

        ## move back to original directory
        os.chdir(self.cwd)


if __name__ == "__main__":
    print "Running..."
    dbf = DatabaseFetch()
    dbf.run()



#!/usr/bin/python
"""
The output from ParallelBlast.py is a large numbers of files.
This is a convenience class for parsing these results as 
we would normally with ParseBlast.py.
"""
import shutil,os,sys,re,csv
import numpy as np
from Bio import SeqIO
from htsint import __basedir__
from htsint.blast import ParseBlast

class ParseParallelBlast(object):
    
    def __init__(self,queryFile,resultsDir=os.path.join(".","cluster")):
        """
        Constructor
        queryFile - is a fasta file of sequences
        resultsDir - is a directory that is purged and filled
        """

        ## variables
        self.queryFile = os.path.realpath(queryFile)
        self.resultsDir = os.path.realpath(resultsDir)
        self.evalue = 0.05

        ## ensure the results directory is present and contains files
        if os.path.isdir(resultsDir) == False or len(os.listdir(resultsDir)) == 0:
            raise Exception("The results directory does not exist")

    def parse_file(self,resultsFilePath):
        """
        use ParseBlast to parse a given xml formatted file
        """

        print '...parsing', resultsFilePath
        parser = ParseBlast(resultsFilePath,fhLog=self.logWriter,fhResults=self.resultsWriter)
        parser.run()

    def run(self,chunks):
        """
        runs ParseBlast on each file and combines the results
        """

        ## variables
        queryFileName = os.path.split(self.queryFile)[-1]
        resultsNameBase = re.sub("\.\w+","",queryFileName,flags=re.IGNORECASE)

        ## prepare a log outfile
        self.fid1 = open(os.path.join(self.resultsDir,resultsNameBase+'_1.log'),'w')
        self.logWriter = csv.writer(self.fid1)
        
        ## prepare a results outfile
        self.fid2 = open(os.path.join(self.resultsDir,resultsNameBase+'_1.csv'),'w')
        self.resultsWriter = csv.writer(self.fid2)
        self.resultsWriter.writerow(["query","hit-identifier","hit-identifier-long","e-score","bit-score"])
        
        ## get the start and stop points
        handleIn = open(self.queryFile, "rU")
        total = 0
        for record in SeqIO.parse(handleIn,"fasta") :
            total += 1
            
        handleIn.close()
        stopPoints = np.arange(0,total,int(round(float(total)/float(chunks))))
        if stopPoints[-1] < total:
            stopPoints = np.hstack([stopPoints[1:],np.array([total])])
     
        ## get the name of the results files
        begin = 0
        for i,chunk in enumerate(range(stopPoints.size)):
            stop = stopPoints[chunk] 
            resultsFile = resultsNameBase + "-%s-%s.xml"%(begin,stop)
            resultsFilePath = os.path.join(self.resultsDir,resultsFile)
            if os.path.exists(resultsFilePath)  == False:
                raise Exception("Cannot find one of the chunked results files -- did you imput the correct number of chunks?")
            self.parse_file(resultsFilePath)
            begin = stop

        ## cleanup 
        self.fid1.close()
        self.fid2.close()
        print "\nParsing completed."
        print "Log file: ", os.path.join(self.resultsDir,resultsNameBase+'_1.log')
        print "Results : ", os.path.join(self.resultsDir,resultsNameBase+'_1.csv')                                 



#!/usr/bin/env python
"""
Assembles the files containing the pairwise distances
that are produced with 'GeneDistances.py' 
"""

__author__ = "Adam Richards"

import os,sys,csv,shutil,re,cPickle,getopt
import numpy as np
import networkx as nx
from basedir import __basedir__

class AssembleDistances(object):
    """
    A generic class
    """

    def __init__(self,termsPath,termGraphPath,resultsDir=os.path.join(".","htsint-tmp"),
                 resultsPath=os.path.join(".","assembled-results.csv")):
        """
        Constructor
        
        if cpus > 1 then the script assumes we are in a cluster envrionment
        Note that the results dir will be cleaned before each run
        """

        ## error checking
        for path in [termGraphPath, termsPath]:
            if os.path.exists(path) == False:
                raise Exception("Cannot find specified path\n%s"%path)
        
        ## variables
        self.termsPath = os.path.realpath(termsPath)
        self.termGraphPath = os.path.realpath(termGraphPath)
        self.baseDir =  os.path.realpath(os.path.dirname(__file__))
            
        ## results dir must contain results
        self.resultsDir = os.path.realpath(resultsDir)

        if re.search("\.csv",resultsPath):
            ## create a files to assemple the results in
            self.writer = csv.writer(open(resultsPath,'w'))
            self.writer.writerow(["i","j","distance"])
        else:
            self.resultsPath = resultsPath

        ## load the term graph and the terms
        self.G = nx.read_gpickle(self.termGraphPath)
        tmp = open(self.termsPath,'r')
        self.gene2go,self.go2gene = cPickle.load(tmp)
        tmp.close()

        ## variables
        self.terms = self.go2gene.keys()
        self.totalTerms = float(len(self.terms))
        self.totalDistances = ((self.totalTerms*(self.totalTerms+1))/2) - self.totalTerms
        self.appendedDistances = 0

    def run(self,name='dist',cpus=1):
        """
        creates bash scripts to be submitted to the cluster queue
        """

        ## check that there are results in the results dir
        if not os.path.isdir(self.resultsDir) or len(os.listdir(self.resultsDir)) == 0:
            raise Exception("There are no results in the results dir \n%s"%self.resultsDir)
                           
        ## variables
        chunks = cpus
        stopPoints = np.arange(0,self.totalDistances,
                               int(round(float(self.totalDistances)/float(chunks))))
        if stopPoints[-1] < self.totalDistances:
            stopPoints = np.hstack([stopPoints[1:],np.array([self.totalDistances])])

        print '...assembling results - %s jobs with %s distances'%(len(stopPoints),self.totalDistances)

        ## create scripts
        mat = np.zeros((self.totalDistances,3),).astype(str)
        begin = 0
        for i,chunk in enumerate(range(stopPoints.size)):
            stop = stopPoints[chunk] 
            submitFile = os.path.join(self.resultsDir,"%s-%s.sh"%(name,i))
            submitLog =  os.path.join(self.resultsDir,"%s-%s.log"%(name,i))       
            
            ## determine if we are using csv or npy
            outFile1 = os.path.join(self.resultsDir,"out-%s-%s.csv"%(int(begin),int(stop)))
            outFile2 = os.path.join(self.resultsDir,"out-%s-%s.npy"%(int(begin),int(stop)))

            if os.path.exists(outFile1):
                self.append_file(outFile1)
            elif os.path.exists(outFile2):
                rows = np.load(outFile2)
                print('adding %s rows...'%(int(stop)-int(begin)))
                self.appendedDistances += (int(stop)-int(begin))
                mat[int(begin):int(stop),:] = rows
            else:
                raise Exception("Results file does not exist -- is the number of cpus correct?\n...%s"%outFile)

            begin = stop

        ## print total
        print("saving...")
        np.save(self.resultsPath,mat)
        print "%s/%s distances appended"%(self.appendedDistances,self.totalDistances)

    def append_file(self,outFile):
        """
        appends a file to the results file
        """
        
        print("...appending %s"%outFile)
        fid = open(outFile,'r')
        reader = csv.reader(fid)
        header = reader.next()

        for linja in reader:
            self.writer.writerow(linja)
            self.appendedDistances += 1

        fid.close()



#!/usr/bin/env python
"""
A generic template

null distribution based on sample permutations



"""

__author__ = "Adam Richards"

class SomeClass(object):
    """
    A generic class
    """

    def __init__(self):
        """
        Constructor

        """










if __name__ == "__main__":
    print "Running..."



#!/usr/bin/env python

import os,sys
import numpy as np
from SilValueGenerator import SilValueGenerator

def get_silhouette_values(matList,matLabelList,subsample=None,minNumEvents=4,resultsType='clusterMeans'):
    '''
    returns a dict of results where files are indexed by a string of the int index
    resultsType -- defines the value of
    example $> silResults = get_silhouette_values([matData1, matData2], [matLabels1, matLabs2])
    silResults 
    '''

    silValues = {}
    silValuesElements = {}
    numFiles = len(matLabelList)
    for expName in range(numFiles):
        silValues[str(expName)] = {}

    ## create subset if data for large data sets 
    subsetExpData = []
    subsetExpLabels = []

    if subsample != None:
        for expInd in range(numFiles):
            expData = matList[expInd]
            expLabels = matLabelList[expInd]
            newIndices = []

            totalInds = 0
            for cluster in np.sort(np.unique(expLabels)):
                clusterInds = np.where(expLabels==cluster)[0]
                totalInds += len(clusterInds)

                if len(clusterInds) > subsample:
                    percentTotal = float(len(clusterInds)) / float(len(expLabels)) 
                    randSelected = np.unique(clusterInds[np.random.randint(0,len(clusterInds),subsample)])
                    newIndices += randSelected.tolist()
                else:
                    newIndices += clusterInds.tolist()

            ## save indices and data
            subsetExpData.append(expData[newIndices,:])
            subsetExpLabels.append(expLabels[newIndices])

    ## calculate the silvalues for each file and the subsampled clusters
    for fileInd in range(numFiles):
            
        if subsample != None:
            fileData = subsetExpData[fileInd]
            fileLabels = subsetExpLabels[fileInd]
        else:
            fileData = matList[fileInd]
            fileLabels = matLabelList[fileInd]

        fileClusters = np.sort(np.unique(fileLabels))    
    
        ## calculate sil values
        svg = SilValueGenerator(fileData,fileLabels)
        silValuesElements[str(fileInd)] = svg.silValues
        
        ## save only sil values for each cluster
        if resultsType == 'clusterMeans':
            for clusterID in fileClusters:
                clusterElementInds = np.where(fileLabels == clusterID)[0]
                if len(clusterElementInds) < minNumEvents:
                    silValues[str(fileInd)][str(clusterID)] = None
                else:
                    clusterSilValue = silValuesElements[str(fileInd)][clusterElementInds].mean()
                    silValues[str(fileInd)][str(clusterID)] = clusterSilValue

                del clusterElementInds
        elif resultsType == 'raw':
            silValues[str(fileInd)] = svg.silValues

    return silValues



#!/usr/bin/env python
"""
A spectral clustering algorithm 

Originally proposed by Andrew Ng et al.

A. Ng, M. Jordan and Y. Weiss 
On spectral clustering: Analysis and an algorithm
In Advances in Neural Information Processing Systems 14, 2001

Modified for self-tuning of sigma and k by Zelnik-Manor et al.

L. Zelnik-manor and P. Perona
Self-tuning spectral clustering
In Advances in Neural Information Processing Systems 17, 2004

"""

__author__ = "Adam Richards"

import sys
import numpy as np
from scipy.spatial.distance import pdist,cdist,squareform
from scipy.cluster.vq import kmeans2
from htsint.stats import get_silhouette_values

class SpectralCluster(object):
    """
    Class to perform spectral clustering per the algorithm by A. Ng.

    M - similarity matrix
    k - the number of clusters
    sigma - bandwidth parameter for the kernel
    similarity - default is a similarity matrix

    """

    def __init__(self,M,dtype='distance'):
        """
        M - matrix 
        dtype - matrix type [points|distance|similarity|]
        """

        ## error checkinn
        dtype = dtype.lower()
        if dtype not in ['points','distance','similarity']:
            raise Exception("Invalid dtype %s"%dtype)

        if dtype == 'points':
            _M = M.copy()
            _M = pdist(_M,'sqeuclidean')
            M = squareform(_M)
            dtype == 'distance'

        if dtype == 'similarity':
            M = np.sqrt(M.max()-M)

        ## add eps to avoid divide by zero
        M = M + np.spacing(1)

        self.M = M

    def run(self,k,sk=7,sigma=None,verbose=False):
        """
        run spectral clustering
        given a number of clusters (k) and bandwidth param (sigma) 
        
        """

        ## create the affinity matrix
        if verbose == True:
            print "\tsimilarity to affinity matrix..."

        k = int(round(k))

        self.A = self.similarity_to_affinity(self.M,sk=sk,sigma=sigma)
        
        ## create the diagonal matrix D
        if verbose == True:
            print "\tcreating diagonal matrix..."

        self.D = np.diag(self.A.sum(axis=1)**-0.5)
        
        ## create the L matrix
        if verbose == True:
            print "\tcreating laplacian matrix..."

        _L = np.dot(self.D,self.A)                      # multiply A and D^{-1/2}
        self.L = np.dot(_L,self.D)                      # multiply the above result times D^{-1/2}
     
        # ensure L contains real, finite numbers
        testNan = np.where(np.isnan(self.L) == True)
        testFinite = np.where(np.isfinite(self.L) == False)

        if np.array([len(z) for z in testFinite]).sum() > 0:
            print "WARNING: failed finite check"
        elif np.array([len(z) for z in testNan]).sum() > 0:
            print "WARNING: failed nan check"

        ## find the k largest eigenvectors of L
        if verbose == True:
            print "\tfinding eigenvalues and eigenvectors..."

        eigVals, eigVecs = np.linalg.eig(self.L)
        eigVecs = -1.0 * eigVecs
        sortedEigInds = np.argsort(np.sum(abs(eigVecs),0))
        self.X = eigVecs[:,sortedEigInds[-k:]]

        ## compute normalized matrix Y from X
        if verbose == True:
             print "\tnormalizing Y matrix..."

        n,k = np.shape(self.X)
        self.Y = np.zeros([n,k])
        unitLengths = np.sum(self.X**2,axis=0)**(0.5)

        for col in range(k):
            self.Y[:,col] = self.X[:,col] / unitLengths[col]

        ## cluster the rows of Y using Kmeans
        if verbose == True:
            print "\tkmeans clustering to get labels..."

        tries = 0
        iters = 0
        minDistortion = 1e08
        bestClusters = None
        bestKmeanLabels = None
        repeats = 10

        bestRepeat = (None,None,-2.0)
        for repeat in range(repeats):
            kmeanResults, kmeanLabels = kmeans2(self.Y,k,minit='points')
            silValues = get_silhouette_values([self.Y],[kmeanLabels],subsample=10000,
                                                      minNumEvents=3,resultsType='raw')
            avgSilVal = silValues['0'].mean()

            if kmeanResults == None or avgSilVal == -2.0:
                continue

            if avgSilVal > bestRepeat[2]:
                bestRepeat = (kmeanResults,kmeanLabels,avgSilVal)

        if bestRepeat[0] == None:
            raise Exception('Kmeans on Y matrix -- did not obtain results')
            return None

        self.labels = bestRepeat[1]
        self.avgSilValue = bestRepeat[2]

        return self.avgSilValue
        
    def similarity_to_affinity(self,dMat,sk=7,sigma=None):
        """
        transform a similarity matrix into an affinity matrix

        """

        ## Ng et al. method
        if sk == None:
            if sigma == None:
                raise Exception("If using Ng et al method must specify sigma")
        
            A = np.exp(-1.0 * (self.M**2.0)  /  2.0 * (sigma**2.0))


        ## using method proposed by Zelnik-Manor et al. method
        else:
            A = np.zeros(dMat.shape)

            if dMat == None:
                print "ERROR: distance matrix is None cannot find affinity"
                return None

            ## get sigma k for each row
            sigmaK = np.zeros(dMat.shape[0])
            for i in range(dMat.shape[0]):
                row = dMat[i,:]
                sigmaK[i] = np.sort(row)[sk-1]

            ## there is prob a faster was of doing this
            for i in range(dMat.shape[0]):
                for j in range(dMat.shape[0]):
                    A[i,j] = np.exp((-1.0 * (dMat[i,j]**2.0))  /  (sigmaK[i] * sigmaK[j]))

        
        ## ensure diag is zeros
        A = A - np.diag(np.diag(A))

        return A


if __name__ == "__main__":
    print "Running..."

    from htsint.stats import dataScatter,dataScatterLabels
    from htsint.stats import dataCircle,dataCircleLabels
    from htsint.stats import dataLetters,dataLettersLabels
    from htsint.stats import dataNetwork,dataNetworkLabels

    import matplotlib.pyplot as plt

    ## setup
    colors = ['cyan','orange','red','green','yellow','magenta','blue','black']
    markers = ["x",".","*","^"]
    fig = plt.figure()
    
    ## scatter plot
    sc = SpectralCluster(dataScatter,dtype='points')
    sc.run(2,sk=7)

    ax = fig.add_subplot(131)
    for c in np.unique(dataScatterLabels):
        inds = np.where(dataScatterLabels == c)[0]
        ax.plot([dataScatter[inds,0]],[dataScatter[inds,1]],marker='o',color=colors[c],markersize=8.0)
    for c in np.unique(sc.labels):
        inds = np.where(sc.labels == c)[0]
        ax.plot([dataScatter[inds,0]],[dataScatter[inds,1]],marker=markers[c],color='k',markersize=8.0)        

    ax.set_aspect(1./ax.get_data_ratio())
    
    ## circle plot
    sc = SpectralCluster(dataCircle,dtype='points')
    sc.run(2,sk=7)

    ax = fig.add_subplot(132)
    for c in np.unique(dataCircleLabels):
        inds = np.where(dataCircleLabels == c)[0]
        ax.plot([dataCircle[inds,0]],[dataCircle[inds,1]],marker='o',color=colors[c],markersize=8.0)
    for c in np.unique(sc.labels):
        inds = np.where(sc.labels == c)[0]
        ax.plot([dataCircle[inds,0]],[dataCircle[inds,1]],marker=markers[c],color='k',markersize=8.0)        

    ax.set_ylim([2.5,7.5])
    ax.set_aspect(1./ax.get_data_ratio())

    ## letters plot
    sc = SpectralCluster(dataLetters,dtype='points')
    sc.run(3,sk=7)

    ax = fig.add_subplot(133)
    for c in np.unique(dataLettersLabels):
        inds = np.where(dataLettersLabels == c)[0]
        ax.plot([dataLetters[inds,0]],[dataLetters[inds,1]],marker='o',color=colors[c],markersize=8.0)
    for c in np.unique(sc.labels):
        inds = np.where(sc.labels == c)[0]
        ax.plot([dataLetters[inds,0]],[dataLetters[inds,1]],marker=markers[c],color='k',markersize=8.0)        

    ax.set_ylim([3,4])
    ax.set_aspect(1./ax.get_data_ratio())

    fig.savefig("spectral-clustering-test.png")



import time,sys
from htsint.database import db_connect,Gene,GoAnnotation,GoTerm
from htsint.database import fetch_annotations,gene_mapper


## variables
session,engine = db_connect()
geneList = ['30970']#,'30971','30972','30973','30975']
expEvidCodes = ["EXP","IDA","IPI","IMP","IGI","IEP"]
compEvidCodes = ["ISS","ISO","ISA","ISM","IGC","RCA"]
statEvidCodes = ["TAS","NAS","IC"]
nonCuratedEvidCodes = ["IEA"]
acceptedCodes = expEvidCodes + statEvidCodes
annotations = {}
aspect = 'biological_process'

timeStart = time.time()
geneQueries = session.query(Gene).filter(Gene.ncbi_id.in_(geneList)).all()
print("...extraction time 1: %s"%time.strftime('%H:%M:%S',time.gmtime(time.time()-timeStart)))

#timeStart = time.time()
#geneIdMap = gene_mapper(session,ncbiIdList=geneList)
#print("...extraction time 1: %s"%time.strftime('%H:%M:%S',time.gmtime(time.time()-timeStart)))

## get all results
timeStart = time.time()
for geneQuery in geneQueries:
    annotations[geneQuery.ncbi_id] = set([])
    print geneQuery.ncbi_id
    annotations[geneQuery.ncbi_id].update(session.query(GoAnnotation).filter_by(gene_id=geneQuery.id).all())
print("...extraction q1: %s"%time.strftime('%H:%M:%S',time.gmtime(time.time()-timeStart)))

for key,items in annotations.iteritems():
    annotations[key] = list(items)
    if None in items:
        annotations[key].remove(None)

for at in annotations['30970']:
    print at,session.query(GoTerm).filter_by(id = at.go_term_id).first().aspect

## get results with table join by aspect
timeStart = time.time()

print 'filtered results'
for geneQuery in geneQueries:
    results = session.query(GoAnnotation).join(GoTerm).\
              filter(GoAnnotation.gene_id==geneQuery.id).\
              filter(GoAnnotation.evidence_code.in_(['ISS'])).\
              filter(GoTerm.aspect=='molecular_function').all()
    
    for r in results:
        print r, session.query(GoTerm).filter_by(id = r.go_term_id).first().aspect



#!/usr/bin/env python
"""
For more information on blast cmd line tools:

    http://www.ncbi.nlm.nih.gov/books/NBK52640/

For more information ofn blast databases

    ftp://ftp.ncbi.nlm.nih.gov/blast/documents/blastdb.html

Some useful information

    blastp  - protein query against a protein database
    blastn  - nucleotide query against a nucleotide database
    blastx  - nucleotide query, dynamically translated in all six frames, against a protein database
    tblastn - protein query against a nucleotide database dynamically translated in all six frames
    tblastx - nucleotide query, dynamically translated in all six frames
              against a nucleotide database similarly translated

    nr      -  non-redundant protein sequence database with entries from GenPept, Swissprot, PIR, 
               PDF, PDB, and NCBI RefSeq
    nt      -  nucleotide sequence database, with entries
               from all traditional divisions of GenBank, 
               EMBL, and DDBJ excluding bulk divisions 
               (gss, sts, pat, est, htg divisions) and wgs 
                entries. Not non-redundant.

An example of blastx usage is:

blastx -query myfile.fa -db nr -out myfile.xml -evalue 0.001 -outfmt 5

"""

import sys,os,time,re,getopt
from Bio import SeqIO
from Bio.Blast.Applications import NcbiblastxCommandline

class Blast(object):
    """
    A generic class to run blast
    """

    def __init__(self,queryFile,BLASTDB="/usr/local/share/htsint"):
        """
        Constructor
            queryFile - is a fasta file of sequences
        """
        
        ## check the query file
        self.queryFile = os.path.realpath(queryFile)
        if os.path.exists(self.queryFile) == False:
            raise Exception("ERROR: Could not locate the fasta file... \n%s"%self.queryFile)

        ## set the environmental variable if specified
        if BLASTDB != None:
            os.environ['BLASTDB'] = BLASTDB

    def get_query_file(self,outDir,start,stop):
        """
        indexing start = 0 and stop = 2 will return a file with
        the first and second sequences in it.

        This means that we start at index start and we stop at 
        the stop index with inclusion.

        """

        queryFileName = os.path.split(self.queryFile)[-1]
        newQueryFile = os.path.join(outDir,re.sub("\.\w+","",queryFileName,flags=re.IGNORECASE)+"-%s-%s.fasta"%(start,stop))
        handleIn = open(self.queryFile, "rU")
        handleOut = open(newQueryFile, "w")
        toWrite = []

        ## get the total number of queries
        

        ## we only include the start if it is 0
        indices = range(start,stop)

        total = -1
        for record in SeqIO.parse(handleIn,"fasta") :
            total += 1
            if total in indices:
                toWrite.append(record)

        SeqIO.write(toWrite,handleOut,'fasta')
        handleIn.close()
        handleOut.close()

        return newQueryFile

    def run_blast(self,targetDB,outDir=".",evalue=0.05,
                   start=None,stop=None,cmd='blastx'):
        """
        targetDB - is the protein database to blast against
        outDir   - a place to put all output files other than cwd
        evalue   - is the blast evalue
        start    - index of the first seq
        stop     - index of the last seq

        """
        
        ## error checking
        outDir = os.path.realpath(outDir)
        if os.path.isdir(outDir) == False:
            raise Exception("Output directory does not exist\n%s"%outDir)
        if start != None and stop == None:
            raise Exception("'start' and 'stop' must be specified together")
        if stop != None and start == None:
            raise Exception("'start' and 'stop' must be specified together")

        outFileName = os.path.split(self.queryFile)[-1]
        outFile = re.sub("\.\w+","",outFileName,flags=re.IGNORECASE)+".xml"

        ## specify the query file and out file
        if start != None or stop != None:
            outFilePath = os.path.join(outDir,re.sub("\.xml","",outFile,flags=re.IGNORECASE)+"-%s-%s.xml"%(start,stop))
            query = self.get_query_file(outDir,start,stop)
        else:
            outFilePath = os.path.join(outDir,outFile)
            query = self.queryFile

        print 'Running blast'
        timeStart = time.time()
        blastx_cline = NcbiblastxCommandline(query=query, db=targetDB, evalue=evalue,
                                             outfmt=5, out=outFilePath,cmd=cmd)
        stdout, stderr = blastx_cline()
        print "Total run time: %s"%time.strftime('%H:%M:%S', time.gmtime(time.time()-timeStart))

if __name__ == "__main__":

    ## read in input file 
    if len(sys.argv) < 3:
        print sys.argv[0] + "-f first -l last -q query_file -d database -e evalue -o outdir"
        sys.exit()

    try:
        optlist, args = getopt.getopt(sys.argv[1:], 'f:l:q:d:e:o:c:')
    except getopt.GetoptError:
        raise Exception(sys.argv[0] + "-f first -l last -q query_file -d database -e evalue -o outdir -c cmd")
        sys.exit()

    first,last,query,database,evalue,cmd = None,None,None,None,None,None
    for o,a in optlist:
        if o == '-f':
            first = int(a)
        if o == '-l':
            last  = int(a)
        if o == '-q':
            query = a
        if o == '-d':
            database  = a
        if o == '-e':
            evalue  = float(a)
        if o == '-o':
            outdir  = a
        if o == '-c':
            cmd  = a

    blast = Blast(query)
    blast.run_blast(database,outDir=outdir,start=first,stop=last,evalue=evalue,cmd=cmd)



#!/usr/bin/python

'''
htsint
config file
'''

__author__ = "A Richards"

import os,csv,re,ast
from version import __version__


defaultCONFIG = {'data':'/usr/local/share/htsint',
                 'dbname':"",
                 'dbuser':"",
                 'dbpass':"",
                 'dbhost':"localhost",
                 'dbport':"5432",
                 'taxa': ['3702','4932','5476','7227','7955','8355','8364','9031','9606',\
                          '10090','10566','10116','28377']
}


class Configure(object):
    """
    class to handle the database config
    """

    def __init__(self):
        """
        constructor
        """

        self.logFileDir = os.path.join(os.path.expanduser('~'),".hts-integrate")
        if os.path.exists(self.logFileDir) == True and os.path.isdir(self.logFileDir) == False:
            os.remove(self.logFileDir)
        
        if os.path.isdir(self.logFileDir) == False:
            os.mkdir(self.logFileDir)

        self.logFilePath = os.path.join(self.logFileDir,"dbconfig.log")

        
        if os.path.exists(self.logFilePath) == False:
            fid = open(self.logFilePath,'w')
            writer = csv.writer(fid)
            
            for key,item in defaultCONFIG.iteritems():
                if item == None:
                    item = 'None'
                elif type(item) != type('i am a string'):
                    item = str(item)

                writer.writerow([key,item])
            fid.close()

        self.log = self.read_project_log(self.logFilePath)
        
    ## effectivly the only action necessary to save a project in its current state
    def save(self):
        writer = csv.writer(open(self.logFilePath,'w'))

        for key,item in self.log.iteritems():
            if item == None:
                item = 'None'
            elif type(item) != type('i am a string'):
                item = str(item)

            writer.writerow([key,item])
            
    ## reads the log file assciated with the current project and returns a dict
    def read_project_log(self,logPathName):
        
        if os.path.isfile(logPathName) == False:
            print "ERROR: invalid model logfile specified",logPathName
            return None
        else:
            logFileDict = {}
            reader = csv.reader(open(logPathName,'r'))
            for linja in reader:

                if re.search("\[|\{|None",str(linja[1])):
                    try:
                        linja[1] = ast.literal_eval(str(linja[1]))
                    except:
                        print 'ERROR: Logger -- string literal conversion failed', linja[1]

                logFileDict[linja[0]] = linja[1]

            return logFileDict



#!/usr/bin/env python
"""
ParseBlast.py 
    takes as input an xml results file
    creates a csv summary results file

    can pass file handles for results files 
    instead of file paths if running over 
    multiple files

    BLAST against the swissprot database and it returns the protein gi number
    BLAST against the nr database and it returns the protein gi number
    
    The identifier in the results is relevant to the target database.

"""

### make imports
import sys,os,re,time,csv,getopt
from Bio.Blast import NCBIXML

class ParseBlast(object):
    """
    parse the xml files for BLAST results
    """

    def __init__(self,filePath,outFile=None,fhResults=None,fhLog=None,BLASTDB=None):
        """
        outFile   - location of where to put results
        fhResults - results file handle (csv.writer)
        fgLog     - log file handle     (csv.writer)

        """

        ## error checking
        if filePath == None or not re.search("\.xml|\.outfmt5",filePath):
            raise Exception("invalid results file -  must be of type xml \n%s"%filePath)

        self.filePath = filePath
        self.fid1,self.fid2 = None,None

        ## prepare a log file
        if fhLog == None:
            resultsHomeDir,fileName = os.path.split(filePath)
            self.fid1 = open(os.path.join(".",'%s_parsed.log'%re.sub("\.xml|\.outfmt5","",fileName)),'w')
            self.logWriter = csv.writer(self.fid1)
        else:
            self.logWriter = fhLog

        ## prepare a summary file
        if fhResults == None:
            if outFile == None:
                outFile = os.path.join('.','%s_parsed.csv'%re.sub("\.xml|\.outfmt5","",fileName))
            self.fid2 = open(outFile,'w')
            self.resultsWriter = csv.writer(self.fid2)
            self.resultsWriter.writerow(["query","hit-identifier","hit-identifier-long","e-score","bit-score"])
        else:
            self.resultsWriter = fhResults

        ## set the environmental variable if specified  
        if BLASTDB != None:
            os.environ['BLASTDB'] = BLASTDB

    def push_out(self,line):
        """
        push a string to both STDOUT and the logfile
        """

        self.logWriter.writerow([line])
        print line

    def run(self):
        """
        run the parser
        currently only saves the best result
        """
        self.push_out(sys.argv[0])
        self.push_out(time.asctime())
        self.push_out("Parsing results file... %s"%self.filePath)

        result_handle = open(self.filePath)
        blast = NCBIXML.parse(result_handle)

        hasResults = 0
        totalResults = 0
        print "parsing... %s"%('0'),
        for record in blast:
            totalResults += 1
            print "\rparsing... %s"%(totalResults),
            if record.alignments:
                query = record.query
                for align in record.alignments:
                    identifier = align.accession
                    hitIdLong = align.title
                    escore = align.hsps[0].expect
                    bitScore =  align.hsps[0].score
                
                    ## remove any commas and write the results
                    self.resultsWriter.writerow([re.sub(",", "", x) for x in [query,str(identifier),hitIdLong]] + [escore,bitScore])
                    hasResults += 1

        self.push_out("total blasted sequences: %s"%totalResults)
        self.push_out("sequences with at least one match : %s"%hasResults)
        self.push_out("percentage of sequences at least one match: %s"%(float(hasResults) / totalResults))

        ## clean up
        if self.fid1 != None:
            self.fid1.close()
        if self.fid2 != None:
            self.fid2.close()
        print 'complete.'



#!/usr/bin/python

import sys,time
from sqlalchemy.sql import select
from htsint.database import db_connect,fetch_annotations,fetch_taxa_annotations
from htsint.database import Taxon,taxa_mapper,Gene,gene_mapper 
session,engine = db_connect()
conn = engine.connect()


#timeStart = time.time()
#annotations = fetch_annotations(['31251'],engine,idType='ncbi',useIea=False,aspect='biological_process')
#print("end: %s"%time.strftime('%H:%M:%S',time.gmtime(time.time()-timeStart)))
#print annotations

##7091(small), 7227(large)
timeStart = time.time()
annotations,goTerms = fetch_taxa_annotations(['7227'],engine,idType='ncbi',useIea=False,aspect='biological_process')
print("end: %s"%time.strftime('%H:%M:%S',time.gmtime(time.time()-timeStart)))
#print annotations



sys.exit()



###########
widget = Gene#Taxon
print("scanning %s"%widget.__tablename__)

timeStart = time.time()
myDict = {}
s = select([widget.id,widget.ncbi_id])
_result = conn.execute(s)  
result = [row for row in _result]
print("core: %s"%time.strftime('%H:%M:%S',time.gmtime(time.time()-timeStart)))
print result[:5]
print result[0]['ncbi_id']
sys.exit()

timeStart = time.time()
for t in session.query(widget).yield_per(5):
    myDict[t.ncbi_id] = t.id
print("yield per: %s"%time.strftime('%H:%M:%S',time.gmtime(time.time()-timeStart)))



"""
statistics classes and functions

"""


from ClusteringLib import get_silhouette_values
from ClusteringData import dataCircle,dataCircleLabels,dataScatter,dataScatterLabels
from ClusteringData import dataLetters,dataLettersLabels,dataNetwork,dataNetworkLabels
from SpectralClustering import SpectralCluster
from SpectralClusterResults import SpectralClusterResults



#!/usr/bin/env python
"""
A class that prints and generates summary information about a species list
with respect to the information contained in the database
"""

__author__ = "Adam Richards"

import csv,re,sys
from htsint.database import db_connect,Taxon,Gene,Uniprot,GoAnnotation,fetch_annotations,get_evidence_codes

class TaxaSummary(object):
    """
    Summarize the information in a taxa list
    """

    def __init__(self,taxaInput,delimiter="\t",upass=''):
        """
        The input is a path to a tab delimited file
        or simply a python list
        The delimiter can also be specified
        The first column is the ncbi taxa id
        All additional columns are optional and user specified 
        """

        self.taxaList = []
        self.session,self.engine = db_connect(upass=upass)
        if type(taxaInput) == type([]):
            for taxon in taxaInput:
                if re.search("\D",taxon):
                    raise Exception("Bad taxa in taxaList (%s)"%(taxon))
                    continue
                self.taxaList.append(taxon)
        else:
            self.taxaList = list(set(self.read_file(taxaInput)))

        print("There are %s unique taxa in the list"%(len(self.taxaList)))

    def print_names_summary(self):
        """
        print to the screen in REstructuredText
        """

        columns = [10,60,40]
        row = "+"
        head = "+"
        for col in columns:
            row += "-"*col+"-+"
            head += "="*col+"=+"

        def show_contents(columns,items,withTrailing=True):
            if len(columns) != len(items):
                raise Exception("Dimension mismatch")

            toPrint = "| "

            for i,item in enumerate(items):
                item = str(item)

                if len(item) >= columns[i]:
                    raise Exception("col %s not large enough min = %s"%(i,len(item)+2))

                toPrint += item+" "*(columns[i]-len(item))+"| "

            print(toPrint[:-1])
            
            if withTrailing:
                print(row)

        print("\nTaxa Summary\n---------------")
        print(row)
        items = ['Taxon ID','Scientific Name','Common Name']
        show_contents(columns,items,withTrailing=False)
        print(head)

        for taxonId in self.taxaList:            
            query = self.session.query(Taxon).filter_by(ncbi_id=taxonId).all()
            if len(query) > 1:
                raise Exception("Taxon id '%s' return multiple values from database"%taxonId)
            if query == None:
                raise Exception("Invalid taxon '%s' provided in list"%taxonId)
            
            show_contents(columns,[query[0].ncbi_id, query[0].name, query[0].common_name_1])

    def print_annotation_summary(self,minCoverage=0.0,useIea=False):
        """
        print a summary of the annotation information
        """
        
        columns = [10,16,18,14,10,10]
        row = "+"
        head = "+"
        for col in columns:
            row += "-"*col+"-+"
            head += "="*col+"=+"

        def show_contents(columns,items,withTrailing=True):
            if len(columns) != len(items):
                raise Exception("Dimension mismatch")

            toPrint = "| "

            for i,item in enumerate(items):
                item = str(item)

                if len(item) >= columns[i]:
                    raise Exception("col %s not large enough min = %s"%(i,len(item)+2))

                toPrint += item+" "*(columns[i]-len(item))+"| "

            print(toPrint[:-1])
            
            if withTrailing:
                print(row)

        print("\nAnnotation Summary\n---------------------")
        print(row)
        items = ['Taxon','genes (annots)','uniprot (annots)','coding-genes','combined','coverage']
        show_contents(columns,items,withTrailing=False)
        print(head)

        for taxon in self.taxaList:
            summary = self.get_annotation_summary(taxon,useIea=useIea)

            if float(summary["coverage"]) < minCoverage:
                continue

            items = [taxon,
                     "%s (%s)"%(summary["num_gene"],summary["num_annotated_genes"]),
                     "%s (%s)"%(summary["num_uniprot"],summary["num_annotated_uniprot"]),
                     summary["num_coding_genes"],
                     summary["num_annotated_combined"],
                     summary["coverage"]]
            show_contents(columns,items)

    def get_annotation_summary(self,taxonId,useIea=False):
        """
        return the annotation summary information for a taxa
        """
        
        acceptedCodes = get_evidence_codes(useIea=useIea)
        taxaQuery = self.session.query(Taxon).filter_by(ncbi_id=taxonId).first()
        annotations = self.session.query(GoAnnotation).filter(GoAnnotation.taxa_id==taxaQuery.id).\
                      filter(GoAnnotation.evidence_code.in_(acceptedCodes)).all()

        geneIds = [g.id for g in self.session.query(Gene).filter_by(taxa_id=taxaQuery.id).all()]
        uniprotQuery = self.session.query(Uniprot).filter_by(taxa_id=taxaQuery.id).all()

        def remove_empty(lst):
            if None in lst:
                lst.remove(None)

        ## get how many genes code for proteins
        codingQuery = self.session.query(Uniprot).filter_by(taxa_id=taxaQuery.id).all()
        codingGenes = list(set([u.gene_id for u in uniprotQuery]))
        remove_empty(codingGenes)
        
        ## get number of genes/proteins with at least one annotation
        annotatedGenes = list(set([a.gene_id for a in annotations]))
        annotatedProts = list(set([a.uniprot_id for a in annotations]))
        remove_empty(annotatedGenes)
        remove_empty(annotatedProts)

        ## remove genes covered that are not of the correct taxa (i.e. viral)
        apQuery = [self.session.query(Uniprot).filter_by(id=uid).first() for uid in annotatedProts]
        _genesFromUniprot = [self.session.query(Gene).filter_by(id=uq.gene_id).first() for uq in apQuery]
        
        if not _genesFromUniprot:
            geneIdsFromUniprot = []
        else:
            remove_empty(_genesFromUniprot)
            genesFromUniprot = []

            for gene in _genesFromUniprot:
                if gene == None:
                    continue
                geneTaxonId = self.session.query(Taxon).filter_by(id=gene.taxa_id).first()
                if int(geneTaxonId.ncbi_id) == int(taxonId):
                    genesFromUniprot.append(gene)
          
            geneIdsFromUniprot = [str(g.id) for g in genesFromUniprot]
        
        annotatedGenes = [str(a) for a in annotatedGenes]

        ## get total genes covered (uniprot + ncbi)
        genesCovered = list(set(annotatedGenes).union(set(geneIdsFromUniprot)))

        ## calculate coverage
        if len(genesCovered) == 0 or len(codingGenes) == 0:
            percentage = 0.0
        else:
            percentage = '%s'%(round((float(len(genesCovered))/float(len(geneIds))) * 100.0,4))

        results = {"num_gene":len(geneIds),
                   "num_uniprot":len(uniprotQuery),
                   "num_coding_genes":len(codingGenes),
                   "num_annotated_genes":len(annotatedGenes),
                   "num_annotated_uniprot":len(annotatedProts),
                   "num_annotated_combined":len(genesCovered),
                   "coverage":percentage}

        return results

    def read_file(self,taxaFilePath):
        """
        Use csv to read in the species 
        """
        
        fid = open(taxaFilePath,'rU')
        reader = csv.reader(fid,delimiter='\t')
        taxaList = []

        for linja in reader:
            if re.search("\D",linja[0]):
                continue
            taxaList.append(linja[0])

        fid.close()

        return taxaList

if __name__ == "__main__":
    print "Running..."

    tsum = TaxaSummary(["8364"])
    tsum.print_names_summary()
    tsum.print_annotation_summary()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os,subprocess, threading

def run_subprocess(cmd):
    proc = subprocess.Popen(cmd,shell=True,stdout=subprocess.PIPE,stdin=subprocess.PIPE)
    while True:
        try:
            next_line = proc.stdout.readline()
            proc.wait()
            if next_line == '' and proc.poll() != None:
                break
        except:
            proc.wait()
            break


class RunSubprocess(object):
    """
    a generic class 
    """

    def __init__(self, cmd, mainWindow=None):
        self.cmd = cmd
        self.mainWindow = mainWindow
        self.process = None
        self.stdOut,self.stdErr = None,None

    def run(self,timeout=100):
        def target():
            self.process = subprocess.Popen(self.cmd,shell=True,stderr=subprocess.PIPE,
                                stdout=subprocess.PIPE,universal_newlines=True,bufsize=4096)

            self.stdOut, self.stdErr = self.process.communicate()

        self.thread = threading.Thread(target=target)
        self.thread.start()

        ## wait a specified amount of time before terminating
        if timeout != None:
            self.thread.join(timeout)
            if self.thread.is_alive():
                print 'The subprocess was auto-terminated due to timeout'
                print "...", self.process.poll()
                self.process.terminate()
                self.thread.join()
        
            return self.process.returncode
        return None

    def add_text(self):
        if self.textScreen == None:
            return
        
        if self.textScreen.showMessages == False:
            self.textScreen.toggle_message_btn()
            
        self.textScreen.add_text(txt)
        QtGui.QApplication.processEvents()

    def terminate(self):
        if self.thread.is_alive():
            self.process.terminate()
            self.thread.join()

if __name__ == '__main__':
    
    myProcess = RunSubprocess("echo 'Process started'; sleep 2; echo 'Process finished'")
    
    ## test should pass
    returnCode = myProcess.run(timeout=10)
    print 'pass return code', returnCode

    ## test should fail
    returnCode = myProcess.run(timeout=1)
    print 'fail return code', returnCode



#!/usr/bin/python

import os,re
import numpy as np



def convert_gene_ids(geneList,target):
    """
    takes a list of gene ids (int) and returns the target field
    Normally, the database can be used, however this script is 
    used for example if a species has not yet been entered in the db.

    """

    if target not in ['taxid','symbol']:
        raise Exception("Invalid target")

    geneInfoFile = os.path.join(os.path.split(os.path.abspath(__file__))[0],"gene_info.db")
    if os.path.exists(geneInfoFile) == False:
        raise Exception("ERROR: cannot find gene info file")

    geneInfoFID = open(geneInfoFile,'rU')
    header = geneInfoFID.next()

    if type(geneList) != type(np.array([])):
        geneList = np.array(geneList)

    toReturn = np.array([None for i in range(len(geneList))])
    found = 0

    for record in geneInfoFID:
        record = record.rstrip("\n")
        record = record.split("\t")
    
        if re.search("^\#",record[0]) or len(record) != 15:
            continue

        geneID = int(record[1])

        #if geneID not in geneList:
        #    continue
        
        indx = np.where(geneList == geneID)[0]
        
        if len(indx) == 0:
            continue
        
        found += 1
        indx = indx[0]

        if target == 'taxid':
            taxID = int(record[0])
            toReturn[indx] = taxID
        if target == 'symbol':
            symbol = record[2]
            toReturn[indx] = symbol

        #LocusTag = record[3]
        #Synonyms = record[4]
        #dbXrefs = record[5]
        #chromosome = record[6]
        #map_location = record[7]
        #description = record[8]
        #type_of_gene = record[9]
        #Symbol_from_nomenclature_authority = record[10]
        #Full_name_from_nomenclature_authority = record[11]
        #Nomenclature_status = record[12]
        #otherDesignations = record[13]
        #Modification_date = record[14]

    toReturn = toReturn.tolist()
    print "convert_gene_ids: %s/%s genes found."%(found,len(toReturn))

    return toReturn


if __name__ == "__main__":


    geneList = [101738635, 692505, 101738735, 101744422, 101267788, 101745190, 101738329, 101739693, 101260649,100808326]
    
    taxList = convert_gene_ids(geneList,'taxid')
    print taxList



#!/usr/bin/env python

"""
tools for expression and count based tasks

"""

import os,sys,csv,gc,re
import numpy as np

def read_RSEM_counts_files(geneFilePath,isoformFilePath):
    """
    read the RSEM counts files into a matrix
    """

    if not os.path.exists(geneFilePath):
        raise Exception("Cannot find gene file\n%s"%(geneFilePath))
    if not os.path.exists(isoformFilePath):
        raise Exception("Cannot find isoform file\n%s"%(isoformFilePath))

    ## load the gene counts
    fid1 = open(geneFilePath,'rU')
    reader1 = csv.reader(fid1,delimiter="\t")
    header1 = reader1.next()    
    results1 = {}
    check = 0
    gc.disable()
    
    for linja in reader1:
        check += 1
        results1[linja[0]] = {'transcript':linja[1],'length':float(linja[2]),'eff_length':float(linja[3]),\
                              'exp_count':int(round(float(linja[4]))),'TPM':float(linja[5]),'FPKM':float(linja[6])}
    fid1.close()
    if check != len(results1.keys()):
        raise Exception("Rows in gene count file are not first columns unique")

    ## load the isoform results
    fid2 = open(isoformFilePath,'rU')
    reader2 = csv.reader(fid2,delimiter="\t")
    header2 = reader2.next()    
    results2 = {}
    check = 0

    for linja in reader2:
        check += 1
        results2[linja[0]] = {'gene':linja[1],'length':float(linja[2]),'eff_length':float(linja[3]),\
                              'exp_count':float(linja[4]),'TPM':float(linja[5]),'FPKM':float(linja[6])}
    fid1.close()
    if check != len(results2.keys()):
        raise Exception("Rows in gene count file are not first columns unique")

    fid2.close()
    gc.enable()

    return results1, results2

def read_matrix(matFilePath,delimiter=",",mtype='float'):
    """
    assumes that row one are the samples and col one are the transcripts
    matrix can only be of mtype 'int' or 'float'

    """

    print 'reading', matFilePath

    if mtype not in ['int','float']:
        raise Exception("mtype must be 'int' or 'float'")
    if not os.path.exists(matFilePath):
        raise Exception("Cannot find matFilePath\n%s"%matFilePath)

    fid = open(matFilePath,'rb')
    reader = csv.reader(fid,delimiter=delimiter)
    header = reader.next()

    ## get the gene and sample ids
    transcriptIds = []
    sampleIds = np.array(header[1:])
    gc.disable()
    for linja in reader:
        transcriptIds.append(linja[0])
    gc.enable()
    transcriptIds = np.array(transcriptIds)
    fid.close()

    ## fill in the matrix
    mat = np.zeros((transcriptIds.shape[0],sampleIds.shape[0]),dtype=mtype)
    fid = open(matFilePath,'rb')
    reader = csv.reader(fid,delimiter=delimiter)
    header = reader.next()
    row = 0 
    for linja in reader:
        if mtype == 'int':
            mat[row,:] = [int(float(i)) for i in linja[1:]]
        else:
            mat[row,:] = [float(i) for i in linja[1:]]
        row +=1
    fid.close()

    return transcriptIds,sampleIds,mat

def read_de_results(filePath,delimiter=",",tool="edgeR"):
    """
    read the differential expression output from DESeq or edgeR

    """

    print 'reading', filePath

    if not os.path.exists(filePath):
        raise Exception("Cannot find matFilePath\n%s"%filePath)

    if tool not in ["edgeR","DESeq"]:
        raise Exception("invalid tool specified use 'edgeR' or 'DESeq'")

    fid = open(filePath,'rb')
    reader = csv.reader(fid,delimiter=delimiter)
    
    ## get columnIds
    header = reader.next()
    columnIds = np.array(header[1:])

    ## get the gene and sample ids
    transcriptIds = []

    gc.disable()
    for linja in reader:
        transcriptIds.append(linja[0])
    gc.enable()
    transcriptIds = np.array(transcriptIds)
    fid.close()

    ## fill in the matrix
    mat = np.zeros((transcriptIds.shape[0],columnIds.shape[0]))
    fid = open(filePath,'rb')
    reader = csv.reader(fid,delimiter=delimiter)
    header = reader.next()
    row = 0 
    for linja in reader:
        _row = [re.sub("NA","NaN",i) for i in linja[1:]]             
        mat[row,:] = [float(i) for i in _row]
        row +=1
    fid.close()

    return transcriptIds,columnIds,mat
    [(x, y) for x in [1,2,3] for y in [3,1,4] if x != y]#

def create_count_matrix(results,label,sampleList):
    """
    this function is untested
    """

    ## use first sample to get rows
    mat = np.zeros((len(results[0].keys()),len(sampleList)))
    keys = sorted(np.array(results[0].keys()))

    for j,sample in enumerate(sampleList):
        for i,key in enumerate(keys):
            mat[i,j] = results[j][key]['exp_count']

    ## write to file 
    fid = open("%s-counts.csv"%label,'w')
    writer = csv.writer(fid)
    if re.search("gene",label):
        writer.writerow(["gene"]+sampleList)
    else:
        writer.writerow(["isoform"]+sampleList)

    for r in range(mat.shape[0]):
        row = [keys[r]] + [int(i) for i in mat[r,:].tolist()]
        writer.writerow(row)

    fid.close()



import sys,os

## database functions and classes
from GeneOntologyLib import read_ontology_file,get_annotation_file,get_ontology_file,get_gene2go_file
from GeneOntologyLib import get_total_annotations,get_evidence_codes,fetch_annotations,get_annotated_genes
from GeneOntologyLib import fetch_taxa_annotations
from DatabaseTables import Base,Taxon,Gene,Uniprot,GoTerm,GoAnnotation
from DatabaseTables import taxa_mapper,gene_mapper,uniprot_mapper,goterm_mapper
from DatabaseTools import get_idmapping_file,get_file_sizes,print_db_summary
from DatabaseTools import ask_upass,db_connect,print_go_summary,read_gene_info_file
from ConversionTools import convert_gene_ids
from DatabaseFetch import DatabaseFetch
from DatabaseCreate import DatabaseCreate



#!/usr/bin/python
"""
break a blast problem into 'chunks' and send the chunks to the cluster
using the pairwise distance problem here

'chunks' may be more than the number of available cpus

currently this is only working with blastx

"""

import shutil,os,sys
import numpy as np
from Bio import SeqIO
from htsint import __basedir__
from htsint.blast import Blast

class ParallelBlast(object):
    
    def __init__(self,queryFile,database,BLASTDB=None,resultsDir=os.path.join(".","cluster"),cmd='blastx'):
        """ 
        Constructor
        queryFile - is a fasta file of sequences
        resultsDir - is a directory that is purged and filled
        """

        ## variables
        self.queryFile = os.path.realpath(queryFile)
        self.database = database
        self.resultsDir = os.path.realpath(resultsDir)
        self.submitFileNames = []
        self.evalue = 0.05
        self.cmd = cmd

        ## ensure the results directory present and clean
        for dirName in [self.resultsDir]:
            if os.path.isdir(dirName):
                shutil.rmtree(dirName)
            os.mkdir(dirName)

    def create_scripts(self,chunks,email,name='blst'):
        """
        chunks - the number of times pieces to break the queue file into
        email  - tells the queue the user email
        name   - a short name (to be seen in the queue)

        """

        ## variables
        script = os.path.join(__basedir__,"blast","Blast.py")

        handleIn = open(self.queryFile, "rU")
        total = 0
        for record in SeqIO.parse(handleIn,"fasta") :
            total += 1
            
        handleIn.close()

        ## break the graph into chunks
        stopPoints = np.arange(0,total,int(round(float(total)/float(chunks))))
        if stopPoints[-1] < total:
            stopPoints = np.hstack([stopPoints[1:],np.array([total])])
        
        print '... submitting %s jobs'%(len(stopPoints))
             
        begin = 0
        for i,chunk in enumerate(range(stopPoints.size)):
            stop = stopPoints[chunk] 
    
            ## make a file for each chunk
            submitFile = os.path.join(self.resultsDir,"%s-%s.sh"%(name,i))
            submitLog =  os.path.join(self.resultsDir,"%s-%s.log"%(name,i))
            args = " -f %s -l %s -q %s -d %s -e %s -o %s -c %s"%(begin,stop,self.queryFile,self.database,
                                                                 self.evalue,self.resultsDir,self.cmd)
            f = open(submitFile, 'w')
            f.write("#!/bin/bash\n" + 
                    "#$ -S /bin/bash\n" +
                    "#$ -j yes\n" +
                    "#S -M %s\n"%email +
                    "#$ -o %s\n"%submitLog + 
                    "/usr/bin/python "+ script + args)
            f.close()
            begin = stop
            self.submitFileNames.append(submitFile)

    def submit(self):
        """
        submit the scripts via qsub
        """
        
        if len(self.submitFileNames) == 0:
            raise Exception("There are no script to be submitted to the queue\n did you run create_scripts?")

        for submitFile in self.submitFileNames:
            os.system("qsub " + submitFile)



"""
blast related classes and functions

"""

from BlastTools import create_blast_map,get_blast_map
from Blast import Blast
from BlastMapper import BlastMapper
from ParallelBlast import ParallelBlast
from ParseBlast import ParseBlast
from ParseParallelBlast import ParseParallelBlast



#!/usr/bin/env python
"""
A generic template
"""

import os,csv
import numpy as np

__author__ = "Adam Richards"

class SpectralClusterResults(object):
    """
    A class to handle spectral clustering results
    """

    def __init__(self,silvalsFile,clustersFile):
        """
        Constructor
        """

        ## error checking
        for filePath in[silvalsFile,clustersFile]:
            if not os.path.exists(filePath):
                raise Exception("could not find file: %s"%filePath)

        self.clusters = self.load_clusters_file(clustersFile)
        self.kRange,self.sigRange,self.silvals = self.load_silvals_file(silvalsFile)

    def load_clusters_file(self,clusterFile):
        """
        load the clusters file
        k,sigma,clustid1,clustid2,....clustid_max

        """
        
        fid = open(clusterFile,'r')
        reader = csv.reader(fid)
        header = reader.next()
        clusterIds = np.array(header[2:])
        results = {}

        for linja in reader:
            k = str(int(linja[0]))
            sigma = str(round(float(linja[1]),4))
            if not results.has_key(k):
                results[k] = {}

            results[k][sigma] = np.array([float(i) for i in linja[2:]])

        fid.close()        
        return results

    def load_silvals_file(self,silvalsFile):
        """
        load the clusters file
        k,sigma,clustid1,clustid2,....clustid_max

        """
        
        fid = open(silvalsFile,'r')
        reader = csv.reader(fid)
        header = reader.next()
        kRange = set([])
        sigRange = set([])
        for linja in reader:
            k = int(linja[0])
            sigma = round(float(linja[1]),4)
            kRange.update([k])
            sigRange.update([sigma])
        fid.close()

        kRange = np.sort(np.array(list(kRange)))
        sigRange = np.sort(np.array(list(sigRange)))

        ## create matrix with k as rows and sigma as columns
        resultsMat = np.zeros((kRange.size,sigRange.size),)

        fid = open(silvalsFile,'r')
        reader = csv.reader(fid)
        header = reader.next()

        for linja in reader:
            k = int(linja[0])
            sigma = round(float(linja[1]),4)

            kInd = np.where(kRange==k)[0]
            sInd = np.where(sigRange==sigma)[0]

            resultsMat[kInd,sInd] = float(linja[2])

        fid.close()

        return kRange,sigRange,resultsMat


if __name__ == "__main__":
    print "Running..."



#!/usr/bin/env python
"""
library of functions for use with the GeneOntology class
"""

import os,sys,re,time
from sqlalchemy.sql import select
from htsint import Configure
from DatabaseTables import Taxon,Uniprot,Gene,GoTerm,GoAnnotation

def remove_empty(lst):
    if None in lst:
        lst.remove(None)

def get_ontology_file():
    """
    check for presence of ontology file
    raise exception when not found
    return the file path
    """

    config = Configure()
    dataDir = config.log['data']
    ontologyFile = os.path.join(dataDir,'go.obo')
    if os.path.exists(ontologyFile) == False:
        raise Exception("Could not find 'go.obo' -- did you run DatabaseFetch?")

    return ontologyFile

def read_ontology_file():
    """
    read the ontology file to find term-term edges
    store all the relationships in a dictionary
    """

    ontologyFile = get_ontology_file()
    fid = open(ontologyFile,'r')
    termCount = 0
    goId = None
    goDict = {"cellular_component":{},
              "molecular_function":{},
              "biological_process":{}}

    def add_term(goNamespace,source,sink):
        if len(sink) != 1 or source == sink[0]:
            return
        if not re.search("GO\:",source) or not re.search("GO\:",sink[0]):
            raise Exception("Invalid go id in ontology file: %s, %s"%(source,sink[0]))

        if goDict[goNamespace].has_key(source) == False:
            goDict[goNamespace][source] = set([])
        goDict[goNamespace][source].update(sink)

    for linja in fid.readlines():
        linja = linja[:-1]

        ## find go id
        if re.search("^id\:",linja):
            goId = re.sub("^id\:|\s+","",linja)
            goName,goNamespace = None,None
            is_a,part_of = None,None
            isObsolete = False
            termCount += 1
            continue 

        ## find a few go id attributes
        if re.search("^name\:",linja):
            goName = re.sub("^name\:\s+","",linja)
        if re.search("^namespace\:",linja):
            goNamespace = re.sub("^namespace\:\s+","",linja)
        if re.search("^def\:",linja):
            goDef = re.sub("^def\:\s+","",linja)
            if re.search("OBSOLETE\.",goDef):
                isObsolete = True

        ## ignore obolete terms
        if goId == None or isObsolete == True:
            continue

        ## add term relationships (is_a)
        if re.search("^is\_a\:",linja):
            is_a = re.sub("^is\_a\:\s+","",linja)
            is_a_sink = re.findall("GO\:\d+",is_a)
            add_term(goNamespace,goId,is_a_sink)

        ## other term relationships
        #if re.search("^relationship\:",linja):
        #    part_of_sink = re.findall("GO\:\d+",linja)
        #    add_term(goNamespace,goId,part_of_sink)
            
    return goDict

def get_annotation_file():
    """
    check for presence of the annotation file
    raise exception when not found
    return the file path
    """

    fileName = 'gene_association.goa_uniprot.db'
    config = Configure()
    dataDir = config.log['data']
    annotationFile = os.path.join(dataDir,fileName)
    if os.path.exists(annotationFile) == False:
        raise Exception("Could not find '%s' -- did you run FetchDbData.py?"%(fileName))

    return annotationFile

def get_total_annotations():
    """
    get the number of annotations in the uniprot file
    this does not include the gene2go file
    """

    config = Configure()
    taxaList = config.log['taxa']

    annotationFile = get_annotation_file()
    annotationFid = open(annotationFile,'rU')
    annotsCount = 0
    annotatedIds = {}
    totalAnnotations = 0

    for record in annotationFid:
        record = record[:-1].split("\t")
        if record[0][0] == "!":
            continue
        if record[0] != 'UniProtKB':
            continue

        taxon = re.sub("taxon:","",record[12])
        if taxon == "" or re.search("\|",taxon):
            continue
        if taxon not in taxaList:
            continue

        totalAnnotations += 1

    gene2goFile = get_gene2go_file()
    gene2goFid = open(gene2goFile,'rU')
    header = gene2goFid.next()

    for record in gene2goFid:
        totalAnnotations += 1

    return totalAnnotations

def get_gene2go_file():
    """
    check for presence of the annotation file
    raise exception when not found
    return the file path
    """

    config = Configure()
    dataDir = config.log['data']
    annotationFile = os.path.join(dataDir,'gene2go.db')
    if os.path.exists(annotationFile) == False:
        raise Exception("Could not find 'gene2go' -- did you run FetchDbData.py?")

    return annotationFile

def get_evidence_codes(useIea=False):
    """
    returns the list of evidence codes to be used by default
    """

    expEvidCodes = ["EXP","IDA","IPI","IMP","IGI","IEP"]
    compEvidCodes = ["ISS","ISO","ISA","ISM","IGC","RCA"]
    statEvidCodes = ["TAS","NAS","IC"]
    nonCuratedEvidCodes = ["IEA"]

    if useIea == False:
        return expEvidCodes + statEvidCodes + compEvidCodes
    else:
        return expEvidCodes + statEvidCodes + compEvidCodes + nonCuratedEvidCodes

def get_annotated_genes(session,taxonId,useIea=False,aspect='biological_process',filterTaxa=True):
    """
    fetch annotations by taxa
    """

    if aspect not in ['biological_process','cellular_component','molecular_function']:
        raise Exception("Invalid aspect specified")

    taxaQuery = session.query(Taxon).filter_by(ncbi_id=taxonId).first()
    acceptedCodes = get_evidence_codes(useIea=useIea)
    annotations = session.query(GoAnnotation).join(GoTerm).\
                  filter(GoAnnotation.taxa_id==taxaQuery.id).\
                  filter(GoAnnotation.evidence_code.in_(acceptedCodes)).\
                  filter(GoTerm.aspect==aspect).all()

    annotatedGenes = list(set([a.gene_id for a in annotations]))
    annotatedProts = list(set([a.uniprot_id for a in annotations]))
    remove_empty(annotatedGenes)
    remove_empty(annotatedProts)
    
    if filterTaxa == False:
        return [session.query(Gene).filter_by(id=gid).first().ncbi_id for gid in annotatedGenes]

    ## remove genes covered that are not of the correct taxa (i.e. viral) 
    apQuery = [session.query(Uniprot).filter_by(id=uid).first() for uid in annotatedProts]
    _genesFromUniprot = [session.query(Gene).filter_by(id=uq.gene_id).first() for uq in apQuery]

    if not _genesFromUniprot:
        geneIdsFromUniprot = []
    else:
        remove_empty(_genesFromUniprot)
        genesFromUniprot = []

        for gene in _genesFromUniprot:
            if gene == None:
                continue
            geneTaxonId = session.query(Taxon).filter_by(id=gene.taxa_id).first()
            if int(geneTaxonId.ncbi_id) == int(taxonId):
                genesFromUniprot.append(gene)

        geneIdsFromUniprot = [str(g.id) for g in genesFromUniprot]

    annotatedGenes = list(set(annotatedGenes).union(set(geneIdsFromUniprot)))
    remove_empty(annotatedGenes)

    return [session.query(Gene).filter_by(id=gid).first().ncbi_id for gid in annotatedGenes]

def fetch_annotations(identifiers,engine,aspect='biological_process',
                      idType='uniprot',useIea=True,verbose=False):
    """
    Fetch the go annotations for a given list of identifiers

    If the identifier is 'uniprot'. Then combine the annotations from that uniportId 
    and the associated geneId if persent.

    If the identifier is 'geneid' the find all uniprot entries and combine all 
    uniprot and geneId results.

    The arg 'asTerms' return

    aspect is 'biological_process', 'cellular_component' or 'molecular_function'

    """

    acceptedCodes = get_evidence_codes(useIea=useIea)
    conn = engine.connect()

    if aspect not in ['biological_process','cellular_component','molecular_function']:
        raise Exception("Invalid aspect specified")

    ## error check
    if type(identifiers) != type([]):
        raise Exception("Takes a list of identifiers")

    annotations = {}
    taxaList = set([])
    idType = idType.lower()
    if idType not in ['uniprot','ncbi']:
        raise Exception("Invalid idType argument in fetch annotations use 'uniprot' or 'ncbi'")

    if idType == 'ncbi':
        if verbose:
            print('fetching annotations')
            print('...translating gene queries')
        s = select([Gene.id,Gene.ncbi_id]).where(Gene.ncbi_id.in_(identifiers))
        _geneQueries = conn.execute(s)
        geneQueries = _geneQueries.fetchall()
        if verbose:
            print("...%s/%s gene queries present"%(len(geneQueries),len(list(set(identifiers)))))
        
        for gquery in geneQueries:
            ncbiId = str(gquery['ncbi_id'])
            annotations[ncbiId] = set([])
            
            ## add results from the gene id
            s = select([GoTerm.go_id,GoTerm.name],GoAnnotation.go_term_id==GoTerm.id).\
                where(GoAnnotation.gene_id == gquery['id']).\
                where(GoAnnotation.evidence_code.in_(acceptedCodes)).\
                where(GoTerm.aspect==aspect)

            _results = conn.execute(s)
            results = [tuple(map(str,r)) for r in _results.fetchall()]
            
            if len(results) > 1:
                annotations[ncbiId].update(results)

            ## check for corresponding uniprot ids
            s = select([Uniprot.id,Uniprot.uniprot_ac,Uniprot.gene_id]).\
                where(Uniprot.gene_id==gquery['id'])
            _uniprotQueries = conn.execute(s)
            uniprotQueries = _uniprotQueries.fetchall()
       
            if len(uniprotQueries) > 0:
                for uquery in uniprotQueries:
                    uniprotAc = str(uquery['uniprot_ac'])
            
                ## add results from the uniprot id
                s = select([GoTerm.go_id,GoTerm.name],GoAnnotation.go_term_id==GoTerm.id).\
                    where(GoAnnotation.uniprot_id == uquery['id']).\
                    where(GoAnnotation.evidence_code.in_(acceptedCodes)).\
                    where(GoTerm.aspect==aspect)
              
                _results = conn.execute(s)
                results = [tuple(map(str,r)) for r in _results.fetchall()]
            
                if len(results) > 1:
                    annotations[ncbiId].update(results)
    
    elif idType == 'uniprot':
        if verbose:
            print('fetching annotations')
            print('...translating uniprot queries')
        s = select([Uniprot.id,Uniprot.uniprot_ac,Uniprot.gene_id]).where(Uniprot.uniprot_ac.in_(identifiers))
        _uniprotQueries = conn.execute(s)
        uniprotQueries = _uniprotQueries.fetchall()
        if verbose:
            print("...%s/%s uniprot queries present"%(len(uniprotQueries),len(list(set(identifiers)))))

        for uquery in uniprotQueries:
            uniprotAc = str(uquery['uniprot_ac'])
            annotations[uniprotAc] = set([])
            
            ## add results from the uniprot id
            s = select([GoTerm.go_id,GoTerm.name],GoAnnotation.go_term_id==GoTerm.id).where(GoAnnotation.uniprot_id == uquery['id']).\
                where(GoAnnotation.evidence_code.in_(acceptedCodes)).\
                where(GoTerm.aspect==aspect)
              
            _results = conn.execute(s)
            results = [tuple(map(str,r)) for r in _results.fetchall()]
            
            if len(results) > 1:
                annotations[uniprotAc].update(results)

            ## add results from the associated gene id
            if uquery['gene_id']:
                s = select([GoTerm.go_id,GoTerm.name],GoAnnotation.go_term_id==GoTerm.id).\
                    where(GoAnnotation.gene_id == uquery['gene_id']).\
                    where(GoAnnotation.evidence_code.in_(acceptedCodes)).\
                    where(GoTerm.aspect==aspect)
              
                _results = conn.execute(s)
                results = [tuple(map(str,r)) for r in _results.fetchall()]

                if len(results) > 1:
                    annotations[uniprotAc].update(results)
        
    ## remove any null results
    for key,items in annotations.iteritems():
        while () in items:
            items.remove(())

        annotations[key] = list(items)

    return annotations


def fetch_taxa_annotations(identifiers,engine,aspect='biological_process',
                           useIea=True,verbose=False):
    """
    Fetch the go annotations for a given list of taxa

    If the identifier is 'uniprot'. Then combine the annotations from that uniportId 
    and the associated geneId if persent.

    If the identifier is 'geneid' the find all uniprot entries and combine all 
    uniprot and geneId results.

    aspect is 'biological_process', 'cellular_component' or 'molecular_function'

    """

    acceptedCodes = get_evidence_codes(useIea=useIea)
    conn = engine.connect()

    if aspect not in ['biological_process','cellular_component','molecular_function']:
        raise Exception("Invalid aspect specified")

    ## error check
    if type(identifiers) != type([]):
        raise Exception("Takes a list of identifiers")

    goTerms = {}
    geneAnnotations = {}
    uniprotAnnotations = {}
    gene2uniprot,uniprot2gene = {},{}
    taxaList = set([])

    ## translate the ncbi taxa ids into db ids
    if verbose:
        print('fetching annotations')
        print('...translating uniprot queries')
    s = select([Taxon.id,Taxon.ncbi_id]).where(Taxon.ncbi_id.in_(identifiers))
    _taxaQueries = conn.execute(s)
    taxaQueries = _taxaQueries.fetchall()
    if verbose:
        print("...%s/%s taxa queries present"%(len(taxaQueries),len(list(set(identifiers)))))

    for tquery in taxaQueries:
        ## get all the annotations for the taxa
        s = select([GoTerm.go_id,GoTerm.name,GoAnnotation.uniprot_id,GoAnnotation.gene_id],GoAnnotation.taxa_id==tquery['id']).\
                    where(GoAnnotation.go_term_id == GoTerm.id).\
                    where(GoAnnotation.evidence_code.in_(acceptedCodes)).\
                    where(GoTerm.aspect==aspect)
        _annotQueries = conn.execute(s)
        annotQueries = _annotQueries.fetchall()
    
        ## get all the gene ids and uniprot ids
        geneIds = []
        uniprotIds = []
        for aq in annotQueries:
            if aq['gene_id']:
                geneIds.append(aq['gene_id'])
            if aq['uniprot_id']:
                uniprotIds.append(aq['uniprot_id'])
            goTerms[str(aq[0])] = str(aq[1])

        ## get the gene to uniprot mappings
        s = select([Gene.id,Gene.ncbi_id],Gene.taxa_id==tquery['id'])
        _geneQueries = conn.execute(s)
        gene2Id = dict([tuple(map(str,r)) for r in _geneQueries.fetchall()])
        
        uniprot2id = {}
        if len(uniprotIds) > 0:
            s = select([Uniprot.id,Uniprot.uniprot_ac,Uniprot.gene_id]).where(Uniprot.id.in_(uniprotIds))
            _uniprotQueries = conn.execute(s)
            uniprotQueries = _uniprotQueries.fetchall()
            for uquery in uniprotQueries:
                uniprotAc = str(uquery['uniprot_ac'])
                uniprot2id[str(uquery['id'])] = uniprotAc
                if uquery['gene_id']:
                    if not gene2uniprot.has_key(str(uquery['gene_id'])):
                        if gene2Id.has_key(str(uquery['gene_id'])):
                            geneNcbi = gene2Id[str(uquery['gene_id'])] 
                            gene2uniprot[geneNcbi] = []
                        else:
                            geneNcbi = None
                    if geneNcbi:
                        gene2uniprot[geneNcbi].append(str(uquery['uniprot_ac']))
                        uniprot2gene[uniprotAc] = geneNcbi
                    else:
                        print "could not find", uquery['gene_id'], 'but we have', uniprotAc

        ## gene centric results
        for aq in annotQueries:
            result = [str(aq[0])]
            if aq[3]:
                geneNcbiId = gene2Id[str(aq[3])]
                if not geneAnnotations.has_key(geneNcbiId):
                    geneAnnotations[geneNcbiId] = set([])
                geneAnnotations[geneNcbiId].update(result)
            if aq[2]:
                uniprotAc = uniprot2id[str(aq[2])]
                if not uniprotAnnotations.has_key(uniprotAc):
                    uniprotAnnotations[uniprotAc] = set([])
                uniprotAnnotations[uniprotAc].update(result)
                
            ## check if we can map gene annotation to uniprot products
            if aq[3]:
                if gene2uniprot.has_key(geneNcbiId) and len(gene2uniprot[geneNcbiId]) > 0:
                    for newUniprotAc in gene2uniprot[geneNcbiId]:
                        if not uniprotAnnotations.has_key(newUniprotAc):
                            uniprotAnnotations[newUniprotAc] = set([])
                        uniprotAnnotations[newUniprotAc].update(result)
            
            ## check if we can map uniprot annotation to gene products
            if aq[2]:
                if uniprot2gene.has_key(uniprotAc):
                    newGeneId = uniprot2gene[uniprotAc]
                    if not geneAnnotations.has_key(newGeneId):
                        geneAnnotations[newGeneId] = set([])
                    geneAnnotations[newGeneId].update(result)

    ## prep the results
    for annotations in [geneAnnotations,uniprotAnnotations]:
        for key,items in annotations.iteritems():
            annotations[key] = list(items)

    return geneAnnotations,uniprotAnnotations

def read_annotation_file():
    """
    read the annotation file into a dictionary
    This will take some time
    This function is intended for use with database population

    http://www.geneontology.org/GO.format.gaf-2_0.shtml
    """

    annotationFile = get_annotation_file()
    annotationFid = open(annotationFile,'rU')
    result = {}

    for record in annotationFid:
        record = record[:-1].split("\t")

        ## check that it is a uniprot entry
        if record[0][0] == "!":
            continue
        
        dbObjectId = record[1]
        dbObjectSymbol = record[2]
        goID = record[4]
        dbReference = record[5]
        evidenceCode = record[6]
        aspect = record[8]
        dbObjectType = record[11]
        taxon = re.sub("taxon:","",record[12])
        date = record[13]
        assignedBy = record[14]

        ## ignore annotations with multiple species
        if re.search("\|",taxon):
            continue

        if not result.has_key(dbObjectId):
            result[dbObjectId] = {'names':set([]),'annots':{},'taxon':taxon}

        result[dbObjectId]['annots'][goID] = [aspect,evidenceCode]
        result[dbObjectId]['names'].update([dbObjectSymbol])

    annotationFid.close()
    return result



#!/usr/bin/env python
"""
Calcuates the pairwise distances between genes 
"""

__author__ = "Adam Richards"

import os,sys,csv,shutil,cPickle,getopt
import numpy as np
import networkx as nx
from basedir import __basedir__

class GeneDistances(object):
    """
    A generic class to handle calculate gene distances using term distances
    """

    def __init__(self,termsPath,termGraphPath,termDistancesPath,outFile=None):
        """
        Constructor
        """

        ## error checking
        for path in [termGraphPath, termsPath]:
            if os.path.exists(path) == False:
                raise Exception("Cannot find specified path\n%s"%path)
        
        ## variables
        self.termsPath = os.path.realpath(termsPath)
        self.termGraphPath = os.path.realpath(termGraphPath)
        self.queue = []
        self.baseDir =  os.path.realpath(os.path.dirname(__file__))
            
        if outFile == None:
            self.outFile = "gdistances.csv"
        else:
            self.outFile = outFile

        ## load the graph, the terms and the term distances
        self.G = nx.read_gpickle(self.termGraphPath)
        tmp = open(self.termsPath,'r')
        self.gene2go,self.go2gene = cPickle.load(tmp)
        tmp.close()

        ## read in the term distances
        termDist = {}
        mat = np.load(termDistancesPath)

        for i in range(mat.shape[0]):
            linja = mat[i,:]

            if not termDist.has_key(linja[0]):
                termDist[linja[0]] = {}
            if not termDist[linja[0]].has_key(linja[1]):
                termDist[linja[0]][linja[1]] = float(linja[2])

        self.termDist = termDist

        ## variables
        self.genes = self.gene2go.keys()
        self.totalGenes = float(len(self.genes))
        self.totalDistances = ((self.totalGenes*(self.totalGenes+1))/2) - self.totalGenes

    def get_min_term_dist(self,sourceTerms,sinkTerms):
        """
        To get the minimum distance between a source and a sink gene
        we use the minimum distance between their go terms
        return the shorest path between two sets of terms
        """

        minDistance = 1e8
        td = 1e8
        for source in sourceTerms:            
            for sink in sinkTerms:
                if source == sink:
                    continue
            
                ## get dist
                if self.termDist.has_key(source) and self.termDist[source].has_key(sink):
                    td = self.termDist[source][sink]
                elif self.termDist.has_key(sink) and self.termDist[sink].has_key(source):
                    td = self.termDist[sink][source]

                if td < minDistance:
                    minDistance = td

        if minDistance == 1e8:
            minDistance = None

        return minDistance


    def run(self,first=None,last=None):
        """
        Search for all pairwise shortest paths
        """

        ## create a results file 
        outFid = open(self.outFile,'wa')
        writer = csv.writer(outFid)
        writer.writerow(["i","j","distance"])

        for i,geneI in enumerate(self.genes):
            termsI = self.gene2go[geneI]
            
            if i % 20 == 0:
                print "%s/%s"%(i,int(self.totalGenes))

            for j,geneJ in enumerate(self.genes):
                termsJ = self.gene2go[geneJ]
                if j >= i:
                    continue
                
                ## find the distance
                distance = self.get_min_term_dist(termsI,termsJ)
                if distance != None:
                    writer.writerow([geneI,geneJ,distance])

        outFid.close()



#!/usr/bin/env python
"""
These are helper scripts to populate the database
"""

### make imports
import sys,os,re,time,csv
import sqlalchemy
import getpass
import numpy as np
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from htsint import Configure
from DatabaseTables import Base,Taxon,Gene,Uniprot,GoTerm,GoAnnotation
from DatabaseTables import taxa_mapper,gene_mapper,uniprot_mapper,goterm_mapper
from htsint.database import get_annotation_file, get_ontology_file, get_gene2go_file

def check_version():
    """
    ensure that sqlalchemy is at least 0.8
    """
    if float(sqlalchemy.__version__[:-2]) < 0.8:
        print "ERROR: SQLAlchemy version is less than required version"
        sys.exit()

def ask_upass():
    """
    returns the pass word for the database
    """


    config = Configure()

    for key in ['data','dbname']:
        if config.log[key] == '':
            raise Exception("You must modify the config file before running DatabaseFetch.py")
    
    check_version()
    upass = config.log['dbpass']
    if upass == '':
        upass = getpass.getpass()

    return upass

def db_connect(verbose=False,upass=''):
    """
    generic function to connect to db

    """

    config = Configure()

    for key in ['data','dbname']:
        if config.log[key] == '':
            raise Exception("You must modify the config file before running DatabaseFetch.py")
    
    check_version()

    ## declare variables
    uname = config.log['dbuser']
    dbhost = config.log['dbhost']
    dbname = config.log['dbname']
    port = config.log['dbport']

    ## get data base parameters
    if upass == '':
        upass = ask_upass()

    if dbname == '' or port == '' or dbhost == '' or uname=='':
        raise Exception("Invalid database parameters -- parameters not specified in config file")
            
    ## create connection to db and create necessary tables 
    print "connecting to database: %s"%dbname
    engine = create_engine('postgresql://%s:%s@%s:%s/%s'%(uname,upass,dbhost,port,dbname),echo=verbose)
    connection = engine.connect()
    Session = sessionmaker(bind=engine)
    session = Session()
    print 'connected.'

    return session,engine

def read_gene_info_file(lineCount=False,short=False):
    """
    read the essential info from NCBI's gene info file
    """

    config = Configure()
    taxaList = config.log['taxa']
    geneInfoFile = os.path.join(config.log['data'],"gene_info.db")
    geneInfoFid = open(geneInfoFile,'rU')
    header = geneInfoFid.next()
    geneInfo ={}
    totalLines = 0

    for record in geneInfoFid:
        record = record.rstrip("\n")
        record = record.split("\t")

        if re.search("^\#",record[0]):
            continue

        taxId = record[0]
        ncbiId = record[1]

        if taxId not in taxaList:
            continue

        if lineCount == True:
            totalLines += 1
            continue

        symbol = record[2]
        synonyms = record[4]
        chromosome = record[6]
        map_location = record[7]
        description = record[8]
        if short == False:
            geneInfo[ncbiId] = [taxId,symbol,synonyms,description]
        else:
            geneInfo[ncbiId] = taxId

    geneInfoFid.close()
    
    if lineCount == True:
        return totalLines
    else:
        return geneInfo

def get_file_sizes():
    """
    return a unique dict of geneids with refseq values from the idmapping file

    """

    config = Configure()
    taxaList = config.log['taxa']
    geneInfoCount = read_gene_info_file(lineCount=True)
    idmappingFile = get_idmapping_file()
    idmappingFid = open(idmappingFile,'rU')
    reader = csv.reader(idmappingFid,delimiter="\t")
    records = set([])
    totalRecords = 0

    for record in reader:
        totalRecords += 1
        if record[1] == 'ncbi-taxa-id' and record[2] not in taxaList:
            continue
        records.update([record[0]])

    idmappingFid.close()
    print 'debug. totalRecords %s, records %s, geneInfoCount %s'%(totalRecords, len(list(records)), geneInfoCount)
    return len(list(records)),geneInfoCount

def populate_taxon_table(engine):
    """
    given a list of taxon ids populate the taxon table    
    populates all taxa
    """
    
    config = Configure()
    namesFile = os.path.join(config.log['data'],"names.dmp")
    if os.path.exists(namesFile) == False:
        raise Exception("Cannot find names.dmp... exiting")

    namesFID = open(namesFile,'rU')
    taxaCount = 0
    timeStart = time.time()
    toAdd = {}
    taxaID = None
    debug = 0

    for linja in namesFID:
        debug += 1

        linja = linja.rstrip("\n")
        linja = linja.split("|")
        linja = [re.sub("\t","",element) for element in linja]

        scientificName,commonName = None,None
        if linja[3] == 'scientific name':
            taxaID = linja[0]
            scientificName = linja[1]
        elif re.search("common name",linja[3]):
            taxaID = linja[0]
            commonName = linja[1]
        else:
            continue

        if taxaID in ['root']:
            continue
                
        ## if record does not exist
        if not toAdd.has_key(taxaID):
            taxaCount += 1
            if len(toAdd) >= 300000:
                with engine.begin() as connection:
                    connection.execute(Taxon.__table__.insert().
                                       values(toAdd.values()))
                toAdd = {}

            toAdd[taxaID] = {'ncbi_id':taxaID,'name':None,'common_name_1':None,
                            'common_name_2':None,'common_name_3':None}
        
        ## if record exists add a common name 
        if toAdd.has_key(taxaID) and scientificName != None:
            toAdd[taxaID]['name'] = scientificName
        elif toAdd.has_key(taxaID) and commonName != None:
            if  toAdd[taxaID]['common_name_1'] == None:
                toAdd[taxaID]['common_name_1'] = commonName
            elif  toAdd[taxaID]['common_name_2'] == None:
                toAdd[taxaID]['common_name_2'] = commonName
            elif  toAdd[taxaID]['common_name_3'] == None:
                toAdd[taxaID]['common_name_3'] = commonName
        else:
            continue

    print('committing changes...')
    with engine.begin() as connection:
        connection.execute(Taxon.__table__.insert().
                           values(toAdd.values()))
    namesFID.close()
    timeStr = "...total time taken: %s"%time.strftime('%H:%M:%S', time.gmtime(time.time()-timeStart))
    addedStr =  "...%s unique taxa were added."%taxaCount
    return timeStr, addedStr

def populate_gene_table(geneInfoCount,session,engine):
    """
    use the geneids derived from the idmapping file along with gene_info data to populate the gene table 
    """

    config = Configure()
    taxaList = config.log['taxa']
    timeStart = time.time()
    toAdd = []
    totalRecords = 0
    total = geneInfoCount
    wayPoints = [round(int(w)) for w in np.linspace(0,total,20)]
    geneInfoFile = os.path.join(config.log['data'],"gene_info.db")
    geneInfoFid = open(geneInfoFile,'rU')
    header = geneInfoFid.next()
    taxaIdMap = taxa_mapper(session)

    for record in geneInfoFid:
        record = record.rstrip("\n")
        record = record.split("\t")

        if re.search("^\#",record[0]):
            continue

        taxId = record[0]

        if taxId not in taxaList:
            continue

        ncbiId = record[1]
        symbol = record[2]
        synonyms = record[4]
        #chromosome = record[6]
        #map_location = record[7]
        description = record[8]
       
        ## define the table entry
        toAdd.append({'ncbi_id':ncbiId,'description':description,
                      'symbol':symbol,'synonyms':synonyms,'taxa_id':taxId})
        totalRecords += 1
        
        if len(toAdd) >= 200000:
            toRemove = []
            for ta in toAdd:
                if taxaIdMap.has_key(ta['taxa_id']):
                    ta['taxa_id'] = taxaIdMap[ta['taxa_id']]
                else:
                    toRemove.append(ta)

            for ta in toRemove:
                toAdd.remove(ta)
                
            if len(toAdd) > 0:
                with engine.begin() as connection:
                    connection.execute(Gene.__table__.insert().
                                       values(toAdd))
            toAdd = []

        ## show progress
        if totalRecords in wayPoints:
            print("\t%s / %s"%(totalRecords,total))

    print('committing changes...')
    toRemove = []
    for ta in toAdd:
        if taxaIdMap.has_key(ta['taxa_id']):
            ta['taxa_id'] = taxaIdMap[ta['taxa_id']]
        else:
            toRemove.append(ta)

    for ta in toRemove:
        toAdd.remove(ta)

    if len(toAdd) > 0:
        with engine.begin() as connection:
            connection.execute(Gene.__table__.insert().
                               values(toAdd))

    ## clean up
    geneInfoFid.close()
    timeStr = "...total time taken: %s"%time.strftime('%H:%M:%S', time.gmtime(time.time()-timeStart))
    addedStr = "...%s unique genes were added."%totalRecords
    return timeStr,addedStr

def populate_uniprot_table(lineCount,session,engine):
    """
    populate the uniprot table with entries from idmappings
    """

    config = Configure()
    taxaList = config.log['taxa']
    timeStart = time.time()
    totalLines,totalRecords = 0,0
    idmappingFile = get_idmapping_file()
    idmappingFid = open(idmappingFile,'rb')
    reader = csv.reader(idmappingFid,delimiter="\t")
    ac2kbMap,toAdd = {},{}
    wayPoints = [round(int(w)) for w in np.linspace(0,lineCount,20)]

    print("getting mappers...")
    geneIdMap = gene_mapper(session)
    taxonIdMap = taxa_mapper(session)
    print("mappers loaded... %s"%time.strftime('%H:%M:%S',time.gmtime(time.time()-timeStart)))

    def queue_entries(toAdd,geneIdMap,taxonIdMap,engine):

        toCommit = []

        for uniprotKbEntry, entry in toAdd.iteritems():
            db_gene_id = None
            db_taxa_id = None
            db_gene_taxa_id = None

            ## convert the gene id to a database key (check old names if we cannot find it)
            if entry['gene-id'] == None:
                pass
            elif geneIdMap.has_key(entry['gene-id']):
                db_gene_id = geneIdMap[entry['gene-id']]
            elif not geneIdMap.has_key(entry['gene-id']):
                _geneIds = [re.sub("\s+","",_ncid) for _ncid in entry['gene-id'].split(";")]
                db_gene_id = None
        
                for _gid in _geneIds:
                    if geneIdMap.has_key(_gid):
                        db_gene_id= _gid

            ## convert the taxa id to a database key
            if entry['ncbi-taxa-id'] and taxonIdMap.has_key(entry['ncbi-taxa-id']):
                db_taxa_id = taxonIdMap[entry['ncbi-taxa-id']]

            ## check that the linked gene taxa is the same as the entry taxa
            #if db_gene_id:
            #    db_gene_taxa_id = session.query(Gene).filter_by(id=db_gene_id).first().taxa_id
            #if db_taxa_id and db_gene_id:
            #    if db_taxa_id != db_gene_taxa_id:
            #        print("WARNING: two taxa present in single uniprot entry? %s %s "%(uniprotKbEntry,\
            #                                                                           entry['gene-id']))

            ## if no taxa was provdied use the one assocated with the linked gene
            #if not db_taxa_id:
            #    dgeneQuery = session.query(Gene).filter_by(id=db_gene_id).first()
            #    if dgeneQuery:
            #        db_taxa_id = dgeneQuery.taxa_id 

            ## ensure we are in appropriate taxa
            if entry['ncbi-taxa-id'] not in taxaList:
                continue

            ## ready the uniprot-ac and refseq rows
            entry['uniprot-ac'] = list(entry['uniprot-ac'])
            if len(entry['uniprot-ac']) == 0:
                entry['uniprot-ac'] = None
            elif  len(entry['uniprot-ac']) == 1:
                entry['uniprot-ac'] = entry['uniprot-ac'][0]
            else:
                entry['uniprot-ac'] = ";".join(entry['uniprot-ac'])

            entry['refseq'] = list(entry['refseq'])
            if len(entry['refseq']) == 0:
                entry['refseq'] = None
            elif  len(entry['refseq']) == 1:
                entry['refseq'] = entry['refseq'][0]
            else:
                entry['refseq'] = ";".join(entry['refseq'])

            ## commit to db
            toCommit.append({'uniprot_ac':entry['uniprot-ac'],'uniprot_entry':uniprotKbEntry,
                             'refseq':entry['refseq'],'taxa_id':db_taxa_id,'gene_id':db_gene_id})
        if len(toCommit) > 0:
            with engine.begin() as connection:
                connection.execute(Uniprot.__table__.insert().
                                   values(toCommit))

    ## parse the idmapping file into the db
    for record in reader:

        if len(record) != 3:
            continue

        uniprotKbAc,uniprotKbEntry,ncbiId,refseq,ncbiTaxaId = None,None,None,None,None
        uniprotKbAc = record[0]
        totalLines += 1
        if totalLines in wayPoints:
            print("\t%s / %s"%(totalLines,lineCount))
        
        if record[1] == 'NCBI_TaxID':
            ncbiTaxaId = record[2]
        elif record[1] == 'GeneID':
            ncbiId = record[2]
        elif record[1] == 'UniProtKB-ID':
            uniprotKbEntry = record[2]
            if not ac2kbMap.has_key(uniprotKbAc):
                ac2kbMap[uniprotKbAc] = uniprotKbEntry
        elif record[1] == 'RefSeq':
            refseq = record[2]
        else:
            continue

        ## skip the XXXX-1 like uniprot ac
        if ac2kbMap.has_key(uniprotKbAc) == False:
            continue

        ## get current key
        uniprotKbEntry = ac2kbMap[uniprotKbAc] 

        ## make new entry if necessary
        if uniprotKbEntry and not toAdd.has_key(uniprotKbEntry):

            ## queue entries in blocks
            totalRecords += 1 
        
            if totalRecords % 100000 == 0:
                queue_entries(toAdd,geneIdMap,taxonIdMap,engine)
                toAdd,ac2kbMap = {},{}


            ac2kbMap[uniprotKbAc] = uniprotKbEntry
            toAdd[uniprotKbEntry] = {'ncbi-taxa-id':None,
                                     'gene-id':None,
                                     'uniprot-ac':set([]),
                                     'refseq':set([])}
        
        ## populate uniprot dictionary
        toAdd[uniprotKbEntry]['uniprot-ac'].update([uniprotKbAc])

        if ncbiTaxaId:
            toAdd[uniprotKbEntry]['ncbi-taxa-id'] = ncbiTaxaId
        elif ncbiId:
            toAdd[uniprotKbEntry]['gene-id'] = ncbiId
        elif refseq:
            toAdd[uniprotKbEntry]['refseq'].update([refseq])

    ## queue any remaining
    if len(toAdd.keys()) > 0:
        queue_entries(toAdd,geneIdMap,taxonIdMap,engine)

    ## clean up
    idmappingFid.close()
    timeStr = "...total time taken: %s"%time.strftime('%H:%M:%S', time.gmtime(time.time()-timeStart))
    addedStr = "...%s unique uniprot entries were added."%totalRecords
    return timeStr,addedStr

def populate_go_terms(engine):
    """ 
    read in the obo file and use it to populate the terms
    """

    timeStart = time.time()
    ontologyFile = get_ontology_file()
    fid = open(ontologyFile,'r')
    termCount = 0
    goId,goName,goNamespace,goDef = None,None,None,None
    toAdd = {}
    isObsolete = False

    for linja in fid.readlines():
        linja = linja[:-1]

        ## find go id
        if re.search("^id\:",linja):
            goId = re.sub("^id\:|\s+","",linja)
            goName,goNamespace,goDef,goAltId = None,None,None,None
            isObsolete = False
            termCount += 1
            toAdd[goId] = {'go_id': goId,'aspect':None,'name':None,
                           'alternate_id':None,'description':None}
            continue

        ## find namespace and description  
        if re.search("^name\:",linja):
            goName = re.sub("^name\:\s+","",linja)
        if re.search("^namespace\:",linja):
            goNamespace = re.sub("^namespace\:\s+","",linja)
        if re.search("^alt_id\:",linja):
            goAltId = re.sub("^alt_id\:\s+","",linja)
        if re.search("^def\:",linja):
            goDef = re.sub("^def\:\s+","",linja)
            if re.search("OBSOLETE\.",goDef):
                isObsolete = True
        
        ## ignore obolete terms
        if isObsolete == True:
            goId = None

        if goId != None:
            if goNamespace != None and toAdd[goId]['aspect'] == None:
                toAdd[goId]['aspect'] = goNamespace
            if goDef != None and toAdd[goId]['description'] == None:
                toAdd[goId]['description'] = goDef
            if goName != None and toAdd[goId]['name'] == None:
                toAdd[goId]['name'] = goName
            if goAltId != None and toAdd[goId]['alternate_id'] == None:
                toAdd[goId]['alternate_id'] = goAltId

    print('committing changes...')
    with engine.begin() as connection:
        connection.execute(GoTerm.__table__.insert().
                           values(toAdd.values()))

    timeStr = "...total time taken: %s"%time.strftime('%H:%M:%S', time.gmtime(time.time()-timeStart))
    addedStr = "...%s unique go term entries were added."%termCount
    return timeStr,addedStr

def populate_go_annotations(totalAnnotations,session,engine):
    """
    read the annotation file into a dictionary
    This will take some time
    This function is intended for use with 
    http://www.geneontology.org/GO.format.gaf-2_0.shtml
    """

    timeStart = time.time()
    config = Configure()
    taxaList = config.log['taxa']
    toAdd = []
    annotationFile = get_annotation_file()
    annotationFid = open(annotationFile,'rU')
    wayPoints = [round(int(w)) for w in np.linspace(0,totalAnnotations,20)]
    annotationCount = 0

    print("...loading mappers")
    termIdMap = goterm_mapper(session)
    taxaIdMap = taxa_mapper(session)
    uniprotIdMap = uniprot_mapper(session)
    print("...populating rows")

    def queue_entry(goId,evidenceCode,pubmedRefs,uniprotId,geneId,taxon,toAdd,mapper,ignoredAnnotations):

        ## remove invalid term ids
        if not termIdMap.has_key(goId):
            queryTerm = session.query(GoTerm).filter_by(alternate_id=goId).first()
            if queryTerm == None:
                return
            go_db_id = queryTerm.id
        else:
            go_db_id = termIdMap[goId]

        ## remove invalid uniprot ids
        if uniprotId and not mapper.has_key(uniprotId):
            return
        if uniprotId:
            uniprot_db_id = mapper[uniprotId]
        else:
            uniprot_db_id = None

        ## remove invalid gene ids
        if geneId and not mapper.has_key(geneId):
            return
        if geneId:
            gene_db_id = mapper[geneId]
        else:
            gene_db_id = None

        ## ignore annotations that have an outdated taxon
        if not taxaIdMap.has_key(taxon):
            ignoredAnnotations += 1
            return

        ## get the taxa foreign key
        taxon_db_id = taxaIdMap[taxon]

        toAdd.append({'go_term_id':go_db_id,'evidence_code':evidenceCode,
                      'pubmed_refs':pubmedRefs,'uniprot_id':uniprot_db_id,
                      'gene_id':gene_db_id,'taxa_id':taxon_db_id})

    ## add annotations from uniprot annotation file
    ignoredAnnotationsUniprot = 0
    print("...getting annotations from gene_association (uniprot)")
    for record in annotationFid:
        record = record[:-1].split("\t")

        ## check that it is a uniprot entry
        if record[0][0] == "!":
            continue
        if record[0] != 'UniProtKB':
            continue
        
        uniprotId = record[1]
        dbObjectSymbol = record[2]
        goId = record[4]
        pubmedRefs = record[5]
        evidenceCode = record[6]
        aspect = record[8]
        uniprotEntry = record[10]
        goTermName = record[11]
        taxon = re.sub("taxon:","",record[12])
        date = record[13]
        assignedBy = record[14]

        if taxon not in taxaList:
            continue

        ## parse the uniprot Entry
        if re.search("\|",uniprotEntry):
            uniprotEntry = re.split("\|",uniprotEntry)[0]

        ## ignore annotations with multiple species
        if re.search("\|",taxon):
            continue

        ## update progress
        annotationCount += 1
        if annotationCount in wayPoints:
            print("\t%s / %s"%(annotationCount,totalAnnotations))

        queue_entry(goId,evidenceCode,pubmedRefs,uniprotEntry,None,taxon,toAdd,
                    uniprotIdMap,ignoredAnnotationsUniprot)

        if len(toAdd) >= 100000: # 100000
            with engine.begin() as connection:
                connection.execute(GoAnnotation.__table__.insert().
                                   values(toAdd))
            toAdd = []

    print('committing final changes...')
    print('ignored annotations after uniprot... %s'%(ignoredAnnotationsUniprot))
    with engine.begin() as connection:
        connection.execute(GoAnnotation.__table__.insert().
                           values(toAdd))

    del uniprotIdMap
    annotationFid.close()
    
    ## add annotations from gene2go
    gene2goFile = get_gene2go_file()
    gene2goFid = open(gene2goFile,'rU')
    ignoredAnnotationsGene = 0 
    print("...getting annotations from gene2go")
    header = gene2goFid.next()
    geneIdMap = gene_mapper(session)
    toAdd = []

    for record in gene2goFid:
        record = record.rstrip("\n")
        record = record.split("\t")

        if re.search("^\#",record[0]) or len(record) != 8:
            continue
    
        taxon = record[0]
        ncbiId = record[1]
        goId = record[2]
        evidenceCode = record[3]
        qualifier = record[4]
        go_term_description = record[5]
        pubmedRefs = record[6]
        go_aspect = record[7]
        annotationCount += 1

        if taxon not in taxaList:
            continue

        if annotationCount in wayPoints:
            print("\t%s / %s"%(annotationCount,totalAnnotations))

        queue_entry(goId,evidenceCode,pubmedRefs,None,ncbiId,taxon,toAdd,
                    geneIdMap,ignoredAnnotationsGene)

        if len(toAdd) >= 100000: #100000
            with engine.begin() as connection:
                connection.execute(GoAnnotation.__table__.insert().
                                   values(toAdd))
            toAdd = []

    print('ignored annotations after gene2go... %s'%(ignoredAnnotationsGene))
    print('committing final changes...')
    
    with engine.begin() as connection:
        connection.execute(GoAnnotation.__table__.insert().
                           values(toAdd))

    timeStr = "...total time taken: %s"%time.strftime('%H:%M:%S', time.gmtime(time.time()-timeStart))
    addedStr = "...%s unique go annotation entries were added."%annotationCount
    return timeStr,addedStr,(ignoredAnnotationsUniprot,ignoredAnnotationsGene)

def print_db_summary():
    """
    print a summary of rows and tables for the database
    """
    
    print('querying database...')
    config = Configure()
    printstr = ""
    session,engine = db_connect(verbose=False)
    printstr += "\nDATABASE - %s - SUMMARY"%config.log['dbname'] + "\n"
    print("\nDATABASE - %s - SUMMARY"%config.log['dbname'])
    for table in [Taxon,Gene,Uniprot,GoTerm,GoAnnotation]:
        print("There are %s entries in the %s table"%(session.query(table).count(),table.__tablename__))
        printstr += "There are %s entries in the %s table"%(session.query(table).count(),table.__tablename__) + "\n"

    return printstr

def get_idmapping_file():
    """
    check for presence of the annotation file
    raise exception when not found
    return the file path 
    """

    config = Configure()
    dataDir = config.log['data']
    idmappingFile = os.path.join(dataDir,'idmapping.dat.db')
    if os.path.exists(idmappingFile) == False:
        raise Exception("Could not find 'idmapping.dat.db' -- did you run FetchDbData.py?")

    return idmappingFile

def print_go_summary(outfile=os.path.join(".","go_summary.csv")):
    """
    print a summary about the organisms with gene ontology information
    """

    session,engine = db_connect()
    goTaxa = []
    fid = open(outfile,'w')
    writer = csv.writer(fid)
    writer.writerow(["ncbi_id","name","common_name","total_genes","total_annotations"])

    for taxa in goTaxa:
        taxQuery = session.query(Taxon).filter_by(ncbi_id=taxa).first()
        geneQuery = session.query(Gene).filter_by(taxa_id=taxQuery.id)
        annotQuery = session.query(GoAnnotation).filter_by(taxa_id=taxQuery.id)
        
        print "..."
        print "TaxID:       %s"%taxQuery.ncbi_id
        print "Species:     %s"%taxQuery.name
        print "Common Name 1: %s"%taxQuery.common_name_1
        print "Common Name 2: %s"%taxQuery.common_name_2
        print "Common Name 3: %s"%taxQuery.common_name_3
        print "Num. Genes:  %s"%geneQuery.count()
        print "Num. GO Annotations:  %s"%annotQuery.count()

        if taxQuery.common_name_1 == '':
            taxQuery.common_name_1 = 'None'

        writer.writerow([taxQuery.ncbi_id,taxQuery.name,taxQuery.common_name_1,
                         geneQuery.count(),annotQuery.count()])

    fid.close()



#!/usr/bin/env python
"""
A class to represent and draw gene sets
"""

__author__ = "Adam Richards"

import os,cPickle,csv,re
import numpy as np
import networkx as nx
import matplotlib as mpl
import matplotlib.pyplot as plt
from htsint.database import db_connect,Gene,Taxon,GoTerm
from htsint.blast import BlastMapper

class GeneSet(object):
    """
    gene set class
    """

    def __init__(self,genesetFile,gene2go,distMat):
        """
        Constructor
        geneList - list
        gene2go - dictionary

        """

        ## setup db
        self.session, self.engine = db_connect()
        self.conn = self.engine.connect()

        ## global variables
        self.dpi = 400
        self.labelOffset = 0.05
        self.fontSize = 8 
        self.legendFontSize = 8
        self.fontName = 'serif'
        self.termAlpha = 0.95
        self.geneAlpha = 0.4
        self.nodeSizeGene = 300
        self.nodeSizeTerm = 300
        self.colors = ['#000000','#FFCC33','#3333DD']
        self.cmap = plt.cm.Blues
        self.lineEnd = 60
        self.linesMax = 32

        ## gene2go input
        if type(gene2go) == type({}):
            self.gene2go = gene2go
        elif os.path.exists(gene2go):
            tmp = open(gene2go,'r')
            self.gene2go,self.go2gene = cPickle.load(tmp)
            tmp.close()
        else:
            raise Exception("Argument 'gene2go' must be dictionary or pickle file path for (gene2go,go2gene)")

        ## geneset file input
        if not os.path.exists(genesetFile):
            raise Exception("Argument 'genesetFile' must be a valid file path")

        self.read_geneset_file(genesetFile)

        ## distance matrix
        if type(distMat) != type(np.array([])):
            raise Exception("Argument 'distMat' must be a NumPy array")
        self.distMat = distMat

    def read_geneset_file(self,genesetFile):
        """
        read in geneset file produced by htsint
        """

        fid = open(genesetFile,'r')
        reader = csv.reader(fid)
        header = reader.next()

        geneset2gene = {}

        if len(header) > 2:
            geneset2transcript = {}
            gene2transcript = {}
            hasTranscripts = True
        else:
            geneset2transcript = None
            gene2transcript = None
            hasTranscripts = False

        for linja in reader:

            ## handle new genesets
            if not geneset2gene.has_key(linja[0]):
                geneset2gene[linja[0]] = set([])
                if hasTranscripts:
                    geneset2transcript[linja[0]] = set([])
            
            ## handle new genes
            if hasTranscripts and not gene2transcript.has_key(linja[1]):
                gene2transcript[linja[1]] = set([])

            ## update the dictionaries
            geneset2gene[linja[0]].update([linja[1]])

            if hasTranscripts:
                geneset2transcript[linja[0]].update(linja[2].split(";"))
            if hasTranscripts:
                gene2transcript[linja[1]].update(linja[2].split(";"))

        ## store as lists
        for key,item in geneset2gene.iteritems():
            geneset2gene[key] = list(item)
        self.geneset2gene = geneset2gene

        if geneset2transcript:
            for key,item in geneset2transcript.iteritems():
                geneset2transcript[key] = list(item)
            self.geneset2transcript = geneset2transcript
        if gene2transcript:
            for key,item in gene2transcript.iteritems():
                gene2transcript[key] = list(item)
            self.gene2transcript = gene2transcript

        fid.close()
        print("-----------------")
        print("%s gene sets loaded from gene set file."%(len(self.geneset2gene.keys())))

    def get_gene_info(self,geneList):
        print("...getting gene info")
        geneInfo = {}
        results = self.conn.execute(Gene.__table__.select(Gene.ncbi_id.in_(geneList)))
        for row in results:
            taxaQuery = self.session.query(Taxon).filter_by(id=row.taxa_id).first()
            geneInfo[str(row.ncbi_id)] = {'symbol': str(row.symbol),
                                          'description': str(row.description),
                                          'taxa': taxaQuery.ncbi_id,
                                          'species': taxaQuery.name
                                      }
        return geneInfo

    def get_term_distances(self,termList,percentile=50):
        """
        return a dictionary of term distances
        ignores any distances greater than percentile threshold
        """

        def scale(val, src, dst):
            """ 
            Scale the given value from the scale of src to the scale of dst. 
            """
            return ((val - src[0]) / (src[1]-src[0])) * (dst[1]-dst[0]) + dst[0]

        print 'getting distances'
        mat = self.distMat[:,2].astype(float)

        ## get the percentile threshold
        threshold = np.percentile(mat,percentile)
        print "percentile threshold: %s (%s)"%(threshold,percentile)

        termDist = {}
        for i in range(self.distMat.shape[0]):
            linja = self.distMat[i,:]
            if linja[0] not in termList or linja[1] not in termList:
                continue
            if float(linja[2]) > threshold:
                continue
            if not termDist.has_key(linja[0]):
                termDist[linja[0]] = {}
            if not termDist[linja[0]].has_key(linja[1]):
                termDist[linja[0]][linja[1]] = 1.0 - scale(float(linja[2]),(mat.min(),mat.max()),(0,1))

        return termDist

    def get_go_term(self,term):
        """
        return a human friendly summary of the go terms
        """

        description =self.session.query(GoTerm).filter(GoTerm.go_id == term).first().name
        if description:
            return description
        else:
            return 'None'

    def draw_network(self,geneSetId,layout='spring',name='None',percentile=50,ax=None):
        """
        create a NetworkX graph 

        allLayouts = ['circular_layout',
                      'random_layout',
                      'shell_layout',
                      'spring_layout',
                      'spectral_layout',
                      'fruchterman_reingold_layout']
        """
                
        if not name:
            pass
        if name == 'None':
            name = "%s_network.png"%(geneSetId)

        if not self.geneset2gene.has_key(geneSetId):
            print("Cannot draw graph for '%s' because ID was not found"%(geneSetId))
            return
        
        print('...fetching information about gene set')
        geneList = self.geneset2gene[geneSetId]

        ## get terms
        geneEdges = []
        termCounts = {}

        for gene in geneList:
            if not self.gene2go.has_key(gene):
                continue
            print "...",gene,self.gene2go[gene]
            for term in self.gene2go[gene]:
                geneEdges.append((gene,term))
                if not termCounts.has_key(term):
                    termCounts[term] = 0
                termCounts[term] += 1

        ## rank the terms by the number of connections
        termList = np.array(termCounts.keys())
        edgeCounts = np.array(termCounts.values())
        rankedInds = np.argsort(edgeCounts)[::-1]
        termIds = np.array(["%s"%str(i+1) for i in range(termList.size)])

        term2id = {}
        for r,rank in enumerate(rankedInds):
            term2id[termList[rank]] = termIds[r]

        termIds = termIds.tolist()    
        geneInfo = self.get_gene_info(geneList)
        taxonNames = list(set([geneInfo[gene]['taxa'] for gene in geneList]))
        taxonNames.sort()
        taxonIds = {}
        for t,taxon in enumerate(taxonNames):
            print t,taxon
            taxonIds[taxon] = str(t)

        geneSymbols = {}
        gene2symbol = {}
        geneLabels = {}
        for gene in geneList:
            nodeId = geneInfo[gene]['symbol'] + "-" + taxonIds[geneInfo[gene]['taxa']]
            taxonId = geneInfo[gene]['taxa']
            gene2symbol[gene] = nodeId
            if not geneLabels.has_key(taxonId):
                geneSymbols[taxonId] = []
                geneLabels[taxonId] = {}
            geneLabels[taxonId][nodeId] = geneInfo[gene]['symbol']
            geneSymbols[taxonId].append(nodeId)

        ## annotation edges
        edgeList1 = [(gene2symbol[edge[0]],term2id[edge[1]]) for edge in geneEdges]
        
        ## term term edges
        termDistances = self.get_term_distances(termList,percentile=percentile)

        print 'term distances...'
        edgeList2 = []
        for term1,items in termDistances.iteritems():
            for term2,dist in items.iteritems():
                edgeList2.append((term2id[term1],term2id[term2],dist))

        ## initialize
        self.G = nx.Graph()
        for taxon in taxonNames:
            for node in geneSymbols[taxon]:
                self.G.add_node(node)
        for term in termIds:
            self.G.add_node(term)
        for edge in edgeList1:
            self.G.add_edge(edge[0],edge[1],weight=3.0)
        edge2colors = []
        for edge in edgeList2:
            self.G.add_edge(edge[0],edge[1],weight=edge[2])
            edge2colors.append(edge[2])
        print("...there are %s annotation edges"%(len(edgeList1)))
        print("...there are %s term-term edges"%(len(edgeList2)))

        ## prepare matplotlib axis
        print('...drawing and saving')

        ## use provided axis or create a new one
        if not ax:
            fig = plt.figure()
            ax = fig.add_subplot(111)
            ax.set_yticks([])
            ax.set_xticks([])

        ## layout
        if layout == 'spring':
            pos = nx.spring_layout(self.G,scale=1)
        elif layout == 'spectral':
            pos = nx.spectral_layout(self.G)
        else:
            print("layout not supported %s... using spring"%(layout))
            pos = nx.spring_layout(self.G)

        ## draw gene nodes for each species
        colorItr = 1

        for taxon in taxonNames:
            symbols = geneSymbols[taxon]
            nodeSizes = [self.nodeSizeGene + (15 * float(len(s))) for s in symbols]
            _G = self.G.subgraph(symbols)
            _pos = {}
            for p in pos:
                if p in symbols:
                    _pos[p] = pos[p]

            nx.draw_networkx_nodes(_G,_pos,node_size=nodeSizes,nodelist=symbols,node_shape=r'o',
                                   node_color=self.colors[colorItr],alpha=self.geneAlpha,ax=ax)
            colorItr += 1

            ## offset labels
            #for p in _pos:
            #    _pos[p][1] += self.labelOffset
            nx.draw_networkx_labels(_G,_pos,font_color='black',font_family=self.fontName,labels=geneLabels[taxon],ax=ax)

        ## draw term nodes
        nx.draw_networkx_nodes(self.G,pos,node_size=self.nodeSizeTerm,nodelist=termIds,node_shape='s',
                               node_color=self.colors[0],alpha=self.termAlpha,ax=ax)
        ## term labels
        G1 = self.G.subgraph(termIds)
        pos1 = {}
        for p in pos:
            if p in termIds:
                pos1[p] = pos[p]
        nx.draw_networkx_labels(G1,pos1,font_color='white',font_family=self.fontName,font_size=self.fontSize,ax=ax)

        ## draw edges
        nx.draw_networkx_edges(self.G,pos,edgelist=edgeList1,width=1.0,edge_color='k',style='dashed',ax=ax,alpha=0.3)
        nx.draw_networkx_edges(self.G,pos,edgelist=edgeList2,edge_color=edge2colors,width=2.0,style='solid',edge_cmap=self.cmap,ax=ax)

        if name:
            plt.savefig(name,bbox_inches='tight',dpi=self.dpi)

        return term2id,termList,taxonNames

    def draw_figure(self,geneSetId,layout='spring',name=None,percentile=50):

        ## specify axes l,b,w,h 
        fig = plt.figure(figsize=(10,6))
        ax1  = fig.add_axes([0.3, 0.0, 0.7, 1.0])
        ax2  = fig.add_axes([0.0, 0.07, 0.3, 0.93],axisbg='#EEEEEE')
        ax3  = fig.add_axes([0.0, 0.0, 0.3, 0.07],axisbg='#EEEEEE')      # empty
        ax4  = fig.add_axes([0.015, 0.03, 0.27, 0.03])

        for ax in [ax1,ax2,ax3,ax4]:
            ax.set_yticks([])
            ax.set_xticks([])
        
        ax2.set_ylim((0,1))
        ax2.set_xlim((0,1))
        #ax1.set_frame_on(False)

        def add_line(toPrint,lineCount,current):
            toPrint = toPrint[:self.lineEnd]
            if lineCount == self.linesMax:
                ax2.text(0.01,current," ...",color='k',fontsize=self.legendFontSize,fontname=self.fontName,ha="left", va="center")
                current = current - increment
                lineCount += 1
                return current, lineCount
            elif lineCount > self.linesMax:
                return current, lineCount

            ax2.text(0.01,current,toPrint,color='k',fontsize=self.legendFontSize,fontname=self.fontName,ha="left", va="center")
            current = current - increment
            lineCount += 1
        
            return current, lineCount

        ## draw the network
        term2id,termList,taxonNames = self.draw_network(geneSetId,layout=layout,name=None,percentile=percentile,ax=ax1)

        ## axis 2 (legend)
        lineCount = 0
        current = 0.98
        increment = 0.03

        ## add the taxa
        #current,lineCount = add_line("taxa 1 goes here",lineCount,current)
        #current,lineCount  add_line("taxa 2 goes here",lineCount,current)        
        
        G1 = nx.Graph()
        colorIter = 1
        pos1 = {}
        for node in taxonNames:
            G1.add_node(node)
            pos1[node] = (0.05,current)
            nx.draw_networkx_nodes(G1,pos1,node_size=100,nodelist=[node],
                                   node_color=self.colors[colorIter],alpha=self.geneAlpha,ax=ax2)
            query = self.session.query(Taxon).filter_by(ncbi_id=node).first()
            toAdd = "        %s (%s)"%(query.name,node)[:self.lineEnd]
            current,lineCount = add_line(toAdd,lineCount,current)
            colorIter += 1
        
        ## intitalize
        """
        G1 = nx.Graph()
        for node in ["A","B","C","1","2","3","4"]:
            G4.add_node(node)

        ## explicitly set edges 
        edgeTerms4 = [("A","B"),("B","C"),("1","2"),("3","4"),("5","6")]
        edges4 = []
        edges4.extend(edgeTerms4)

        for edge in edges4:
            G4.add_edge(edge[0],edge[1],weight=1)

        ## explicitly set positions
        pos4={"A":(0.15,0.9),
              "B":(0.15,0.74),
              "C":(0.15,0.58),
              "1":(0.05,0.42),
              "2":(0.25,0.42),
              "3":(0.05,0.26),
              "4":(0.25,0.26),
              "5":(0.05,0.1),
              "6":(0.25,0.1)}
        """

        ## add all of the go lines
        current,lineCount = add_line("-"*self.lineEnd,lineCount,current)
        termIds = [int(i) for i in list(set(term2id.values()))]
        termIds.sort()
        termIds = [str(i) for i in termIds]
        id2term = {}
        for key,item in term2id.iteritems():
            id2term[item] = key
        
        for termId in termIds:
            term = id2term[termId]
            desc = self.get_go_term(term)
            toPrint = "%s - %s"%(termId.zfill(3),desc)
            print toPrint, term
            
            ## check to see if we need to use multiple lines
            if len(toPrint) > self.lineEnd:
                wordBreaks = np.array([m.start(0) for m in re.finditer("\s+",toPrint)])
                lineBreak = wordBreaks[np.where(wordBreaks < self.lineEnd)[0]][-1]
                remaining = toPrint[lineBreak:]
                toPrint = toPrint[:lineBreak]
            else:
                remaining = None
            toPrint = toPrint[:self.lineEnd]
            current,lineCount = add_line(toPrint,lineCount,current)

            if remaining:
                current,lineCount = add_line("        "+remaining,lineCount,current)

        ## axis 4 (colorbar)
        norm = mpl.colors.Normalize(vmin=0.0, vmax=1.0)
        cbar = mpl.colorbar.ColorbarBase(ax4,cmap=self.cmap,
                                         ticks=[0.0,0.25,0.5,0.75,1.0],
                                         norm=norm,
                                         orientation='horizontal')
        cbar.ax.tick_params(labelsize=self.fontSize) 

        ## save
        plt.savefig(name,bbox_inches='tight',dpi=self.dpi)


if __name__ == "__main__":
    print "Running..."



#!/usr/bin/env python

import os,sys,csv,re
from sqlalchemy.sql import select
from htsint.database import db_connect,Taxon,Gene


def get_blast_map(resultsFilePath,evalue=0.00001,taxaList=None,asGenes=False,append=False):
    """
    load assembly blast results into dictionary

    if taxaList is provided then only genes from given taxa will be included in map

    if asGene == True the results are provided with keys to genes not isoforms


    """

    if not os.path.exists(resultsFilePath):
        raise Exception("cannot find results file path %s"%resultsFilePath)

    if taxaList != None:
        ## prepare database connections
        session,engine = db_connect()
        conn = engine.connect()
        s = select([Taxon.id,Taxon.ncbi_id,Taxon.name]).where(Taxon.ncbi_id.in_(taxaList))
        _taxaQueries = conn.execute(s)
        taxaQueries = _taxaQueries.fetchall()
        totalQueries = set([])
        filteredQueries = set([])
        filteredHits = set([])
        selectedTaxa = [str(tquery['id']) for tquery in taxaQueries]
        taxa2name = dict([(str(tquery['id']),str(tquery['ncbi_id'])) for tquery in taxaQueries])
        
        ## create a gene2taxa dictionary
        #gene2taxa,gene2desc = {},{}
        #for tquery in taxaQueries:
        #    s = select([Gene.taxa_id,Gene.ncbi_id,Gene.description],Gene.taxa_id==tquery['id'])
        #    _geneQueries = conn.execute(s)
        #    geneQueries = _geneQueries.fetchall()
        #    gene2taxa.update(dict([(str(r['ncbi_id']),str(r['taxa_id'])) for r in geneQueries]))
        #    gene2desc.update(dict([(str(r['ncbi_id']),str(r['description'])) for r in geneQueries]))

    results = {}
    fid = open(resultsFilePath,'rU')
    reader = csv.reader(fid)
    header = reader.next()
    print header

    ## loop through file and save best
    uniqueQueries = set([])
    totalQueries = 0
    unfilteredQueries = 0

    for linja in reader:
        
        if len(linja) == 4:
            queryId = linja[0]
            hitId = linja[1]
            hitNcbiId = linja[2]
            _evalue = float(linja[3])
        else:
            queryId = linja[0]
            queryNcbi = linja[1]
            hitId = linja[2]
            hitNcbiId = linja[3]
            _evalue = linja[4]

        if asGenes == True:
            queryId = re.sub("_i\d+","",queryId)

        # filtering
        totalQueries += 1
        if '-' in linja:
            continue
        if _evalue > evalue:
            continue

        if taxaList and gene2taxa.has_key(str(hitNcbiId)) == False:
            continue

        unfilteredQueries += 1
        uniqueQueries.update([queryId])
        
        ## use the best evalue
        if not results.has_key(queryId):
            if append:
                results[queryId] = [(hitNcbiId,_evalue)]
            else:
                results[queryId] = (hitNcbiId,_evalue)
        if _evalue < results[queryId][1]:
            if append:
                results[queryId].append((hitNcbiId,_evalue))
            else:
                results[queryId] = (hitNcbiId,_evalue)
                
    uniqueQueries = list(uniqueQueries)
    
    print("total queries: %s"%totalQueries)
    print("unfiltered queries: %s"%unfilteredQueries)
    print("unique: %s"%len(uniqueQueries))
        
    return results


def create_blast_map(refTaxon,taxaList,resultsFilePath,evalue=0.00001,verbose=False):
    """
    read a summarized reference blast results file and create a map
    results are gene centric

    example results file looks like this
    query(refseq),query(geneId),hit(uniprotEntry),hit(geneEntry),e-value
    NP_001016845.1,549599,AQP3_HUMAN,360,1.24637e-170
    NP_001016845.1,549599,AQP9_HUMAN,366,7.57313e-92
    NP_001016845.1,549599,AQP10_HUMAN,89872,2.87154e-85
    NP_001016845.1,549599,AQP7_HUMAN,364,8.01308e-84
    NP_001016845.1,549599,AQP5_HUMAN,362,4.267e-15

    """

    ## error check
    if not os.path.exists(resultsFilePath):
        raise Exception("cannot find results file path %s"%resultsFilePath)

    if refTaxon not in taxaList:
        raise Exception("refTaxon must be in taxaList")

    ## prepare database connections
    session,engine = db_connect()
    conn = engine.connect()

    ## read through the file to map the genes to taxa ids    
    s = select([Taxon.id,Taxon.ncbi_id,Taxon.name]).where(Taxon.ncbi_id.in_(taxaList))
    _taxaQueries = conn.execute(s)
    taxaQueries = _taxaQueries.fetchall()
    totalQueries = set([])
    filteredQueries = set([])
    filteredHits = set([])
    selectedTaxa = [str(tquery['id']) for tquery in taxaQueries]
    taxa2name = dict([(str(tquery['id']),str(tquery['ncbi_id'])) for tquery in taxaQueries])

    ## create a gene2taxa dictionary
    gene2taxa = {}
    for tquery in taxaQueries:
        s = select([Gene.taxa_id,Gene.ncbi_id],Gene.taxa_id==tquery['id'])
        _geneQueries = conn.execute(s)
        gene2taxa.update(dict([(str(r['ncbi_id']),str(r['taxa_id'])) for r in _geneQueries.fetchall()]))

    ## creats a dictionary results['geneId]['taxaId'] = bestHitMappedTaxa
    results = {}
    fid = open(resultsFilePath,'rU')
    reader = csv.reader(fid)
    header = reader.next()
        
    ## loop through file and save best
    for linja in reader:
        _evalue = float(linja[4])
        totalQueries.update([linja[1]])

        ## filter by species and evalue
        if not gene2taxa.has_key(linja[3]) or _evalue > evalue:
            continue
        if linja[1] == '-' or linja[3] == '-':
            continue

        ## filter self matches
        if taxa2name[gene2taxa[linja[3]]] == refTaxon:
            continue
        if taxa2name[gene2taxa[linja[1]]] != refTaxon:
            raise Exception("Invalid query or invalid refTaxon %s != %s"%(taxa2name[gene2taxa[linja[1]]],refTaxon))
    
        taxId = taxa2name[gene2taxa[linja[3]]]
        filteredQueries.update([linja[1]])
        filteredHits.update([linja[3]])
        
        if not results.has_key(taxId):
            results[taxId] = {}
        
        ## use the best evalue
        if not results[taxId].has_key(linja[1]):
            results[taxId][linja[1]] = (linja[3],_evalue)
        if _evalue < results[taxId][linja[1]][1]:
            results[taxId][linja[1]] = (linja[3],_evalue)

    fid.close()
    
    ## returns a simplified form of the results as two mappers
    mapper1,mapper2 = {},{}
    test1 = set([])
    for taxId in results.iterkeys(): 
        for queryGene,hit in results[taxId].iteritems():
            if not mapper1.has_key(hit[0]):
                mapper1[hit[0]] = []
            mapper1[hit[0]].append(queryGene)
            
            if not mapper2.has_key(queryGene):
                mapper2[queryGene] = []
            mapper2[queryGene].append(hit[0])

    #debug = set([])
    #for key, item in mapper.iteritems():
    #    debug.update(item)
    
    #for tquery in taxaQueries:
    #    taxId = str(tquery['ncbi_id'])
    #    for key,item in results[taxId].iteritems():
    #        debug.update([key])
    #        mapper[item[0]] = key

    #debug = list(debug)
    #print 'debug', len(debug),missing
    #print results.keys(),len(results['8364'].keys()), len(results['8355'].keys()), len(list(set(results['8364'].keys() + results['8355'].keys())))
    print('BLAST: total queries: %s'%(len(list(totalQueries))))
    print('BLAST: filtered queries (evalue=%s)(taxa=%s): %s'%(evalue,str(taxaList),len(list(filteredQueries))))
    print('BLAST: filtered hits: %s'%(len(list(filteredHits))))
    return mapper1,mapper2
